
/*****************************************************************
Copyright 1991 by Sony Microsystems Company, San Jose, California

					All Rights Reserved

Permission to use, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and
that both that copyright notice and this permission notice appear
in supporting documentation, and that the name of Sony not be used
in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.

SONY DISCLAIMS ANY AND ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
INCLUDING ALL EXPRESS WARRANTIES AND ALL IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS, FOR A PARTICULAR PURPOSE. IN NO EVENT
SHALL SONY BE LIABLE FOR ANY DAMAGES OF ANY KIND, INCLUDING BUT NOT
LIMITED TO SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES RESULTING FROM
LOSS OF USE, DATA OR LOSS OF ANY PAST, PRESENT, OR PROSPECTIVE PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIENCE OR OTHER TORTIOUS ACTION, 
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS 
SOFTWARE.
*****************************************************************/


/*
 *
 *
 */

#include <stdio.h>
#include <math.h>
#include <errno.h>
#include <X11/Xlib.h>
#include <X11/Xatom.h>
#include <X11/Xutil.h>
#include "Xpexlib.h"
#include "Xpexutil.h"
#include "window.h"

/* #define AUTO_BOX_DEBUG */
#ifdef AUTO_BOX_DEBUG
#define Print(form) (void) fprintf form
#else
#define Print(form)
#define print_structure_info(a,b)
#define print_matrix(a)
#endif

#ifndef M_PI
#define M_PI 4 * atan(1.0);
#endif /* M_PI */

#define BORDER_WIDTH   2

static Display *dpy;
static Window drawable;
static XpexWksResources wks_res, *pWksRes = &wks_res;

/* Globals */

XpexCoord3D   pt;
XpexVector3D  shift;
XpexFloat     x_ang, y_ang, z_ang;
XpexVector3D  scale;
XpexMatrix    bldmat, bldmat2, bldmat3, bldmat4;

XpexPhigsWks   wks_1;
XpexStructure  root;
XpexStructure  cube;
XpexStructure  sub;

#define BLACK	0
#define WHITE	1
#define RED		2
#define GREEN	3
#define BLUE	4
#define YELLOW	5
#define CYAN	6
#define MAGENTA 7

/* Labels */
#define STYLE   1 
#define QUAD_1  101 
#define QUAD_2 	102 
#define QUAD_3 	103 
#define QUAD_4	104 
#define QUAD_1_1 105
#define QUAD_4_1 106

/* #define USE_FILL_AREA_SET_3 */

#ifdef USE_FILL_AREA_SET_3
#define FILL_AREA_CALL(DPY,P,N) \
{ \
	XpexListOfCoord3D list; \
	list.num_points = N; \
	list.points = P; \
	XpexFillAreaSet(DPY, Xpex_UnknownShape,True, Xpex_UnknownContour, \
	&list,1); \
}
#else /* USE_FILL_AREA_SET_3 */
#define FILL_AREA_CALL(DPY,P,N) \
XpexFillArea(DPY,Xpex_UnknownShape,True,P,N)
#endif /* USE_FILL_AREA_SET_3 */

#define SET_ELEM_PTR_TO_BEGINNING(_DPY,_SID) \
{ \
	static XpexElementPos elem_pos = { \
		Xpex_Beginning, 0 \
	}; \
	XpexSetElementPointer(_DPY,_SID,&elem_pos); \
}

#define DELETE_ELEM(_DPY,_SID) \
{ \
	static XpexElementRange elem_range = { \
		{ Xpex_Current, 0 }, \
		{ Xpex_Current, 0 } \
	}; \
	XpexDeleteElements(dpy, root, &elem_range); \
}

#ifdef AUTO_BOX_DEBUG
static void
print_structure_info(dpy, sid)
	Display *dpy;
	XpexStructure sid;
{
	XpexStructureInfo info;

	XpexGetStructureInfo(dpy, sid, &info);

	Print((stderr,"s_id: %d\n", sid));
	Print((stderr,"editing_mode: %s\n", 
		info.editing_mode ? "Replace" : "Insert" ));
	Print((stderr,"element_ptr: %d\n", info.element_ptr));
	Print((stderr,"num_elements: %d\n", info.num_elements));
	Print((stderr,"total_length: %d\n", info.total_length));
	Print((stderr,"has_refs: %d\n", info.has_refs));
}


static void
print_matrix( mat )
    XpexFloat mat[4][4];
{
    register long i;

    for( i = 0; i <=3; i++ ) 
        Print((stderr, "[%f %f %f %f ]\n", 
	    mat[0][i], mat[1][i], mat[2][i], mat[3][i]));
}
#endif

static void
setup(n, mat)
int n;
XpexFloat mat[4][4];
{
    int i, j;

    for (i = 0; i < n; i++) {
        for (j = 0; j < n; j++) {
            mat[i][j] = 0.0;
            if (i == j) mat[i][j] = 1.0;
        }
    }
} 

static void
rotate_views()
{
	float	theta;
	int		err;

    Print((stderr, "rotate_views(); IN\n"));
/*
 * ... enable editing in the replace mode -- this will be used to update
 * rotation angles in 4 views
 */
    XpexSetDisplayUpdateMode(dpy, wks_1, Xpex_VisualizeEasy);
    XpexSetEditingMode(dpy, sub, XpexStructure_Replace);
/*
 * ...  set some constants for building the matrix
 */        
    pt.x = 10.; pt.y = 10.; pt.z = 10.;
    shift.x = 0.0; shift.y = 0.0; shift.z = 0.0;
    scale.x = 1.0; scale.y = 1.0; scale.z = 1.0;
/*
 * ...  make identity matrices
 */
    setup(4, bldmat);
    setup(4, bldmat2);
    setup(4, bldmat3);
    setup(4, bldmat4);
/*
 * ...  edit the structure
 */

/*
 * ... increment the angle in a postive angle
 */     

    for (theta = 0; theta < 4 * M_PI; theta += 0.4) {
/*
 * ...  set the pointer and the angles, 
 * ...	find the label, 
 * ...	build the matrix, 
 * ...	update the view
 */
		SET_ELEM_PTR_TO_BEGINNING(dpy, sub);
/*
 * ... viewport 1
 */
        x_ang = theta;
        y_ang = 0.0;
        z_ang = 0.0;

		XpexSetElementPointerAtLabel(dpy, sub, 1, 1);

        XpexBuildTransformationMatrix(&pt, &shift, 
		x_ang, y_ang, z_ang, &scale, bldmat);

#ifdef AUTO_BOX_DEBUG
	print_matrix(bldmat);
#endif /* APPL_DEBUG */
	XpexStoreOutputCmds(dpy, sub);
	XpexSetLocalTransform(dpy, Xpex_Replace, bldmat);

/*
 * ... viewport 2
 */
	x_ang = 0;
	y_ang = theta;

	XpexSetElementPointerAtLabel(dpy, sub, 2, 1);

	XpexBuildTransformationMatrix(&pt, &shift, 
	x_ang, y_ang, z_ang, &scale, bldmat2);

	/* XpexStoreOutputCmds(dpy, sub); */
	XpexSetLocalTransform(dpy, Xpex_Replace, bldmat2);

/*
 * ... viewport 3
 */

       	y_ang = 0;
       	z_ang = theta;

	XpexSetElementPointerAtLabel(dpy, sub, 3, 1);

	XpexBuildTransformationMatrix(&pt, &shift, 
	x_ang, y_ang, z_ang, &scale, bldmat3);

	/* XpexStoreOutputCmds(dpy, sub); */
	XpexSetLocalTransform(dpy, Xpex_Replace, bldmat3);

/*
 * ... viewport 4
 */
	x_ang = theta;
	y_ang = theta;
	z_ang = theta;

	XpexSetElementPointerAtLabel(dpy, sub, 4, 1);

	XpexBuildTransformationMatrix(&pt, &shift, 
	x_ang, y_ang, z_ang, &scale, bldmat4);

	/* XpexStoreOutputCmds(dpy, sub); */
	XpexSetLocalTransform(dpy, Xpex_Replace, bldmat4);

/*	XFlush( display); */
/*
 * pupd_ws (wks_1, PPERFORM);
 *
 *    XPEXExecuteDeferredActions
 *
 *    XPEXUpdateWorkstation
 *
 *
 */

	    XpexExecuteDeferredActions(dpy, wks_1);

		XpexUpdateWorkstation(dpy, wks_1);

	    XFlush(dpy);

	}
    Print((stderr, "rotate_views(); OUT\n"));
}

static void
scale_views( scale_factor)
    float scale_factor;
{
    float           i;
    int             err;

/*
 * ... enable editing in the replace mode -- this will be used to update
 *                                       the scale matrix in 4 views
 */
    XpexSetDisplayUpdateMode(dpy, wks_1, Xpex_VisualizeEasy);

    XpexSetEditingMode(dpy, sub, XpexStructure_Replace);
/*
 * ...  open the structure for editing
 */
/*
 * ... increment the scale matrix 
 */     

    for (i = 1.0; i < 4.0; i += .2) {
/*
 * ...  set the pointer and the angles, find the label, build the matrix, update
 * ...  the view
 */
		SET_ELEM_PTR_TO_BEGINNING(dpy, sub);
/*
 * ... viewport 1
 */
        scale.x = scale.x * scale_factor; 
        scale.y = scale.y * scale_factor; 
        scale.z = scale.z * scale_factor;
	
        XpexSetElementPointerAtLabel(dpy, sub, 1, 1);

        XpexBuildTransformationMatrix(&pt, &shift, 
		x_ang, y_ang, z_ang, &scale, bldmat);

		XpexStoreOutputCmds(dpy, sub);
        XpexSetLocalTransform(dpy, Xpex_Replace, bldmat);

/*
 * ... viewport 2
 */
	    XpexSetElementPointerAtLabel(dpy, sub, 2, 1);

        XpexBuildTransformationMatrix(&pt, &shift, 
		x_ang, y_ang, z_ang, &scale, bldmat2);

		/* XpexStoreOutputCmds(dpy, sub); */
        XpexSetLocalTransform(dpy, Xpex_Replace, bldmat2);
/*
 * ... viewport 3
 */
        XpexSetElementPointerAtLabel(dpy, sub, 3, 1);

        XpexBuildTransformationMatrix(&pt, &shift, 
		x_ang, y_ang, z_ang, &scale, bldmat3);

		/* XpexStoreOutputCmds(dpy, sub); */
        XpexSetLocalTransform(dpy, Xpex_Replace, bldmat3);
/*
 * ... viewport 4
 */

        XpexSetElementPointerAtLabel(dpy, sub, 4, 1);

        XpexBuildTransformationMatrix(&pt, &shift, 
		x_ang, y_ang, z_ang, &scale, bldmat4);

		/* XpexStoreOutputCmds(dpy, sub); */
        XpexSetLocalTransform(dpy, Xpex_Replace, bldmat4);

        XpexExecuteDeferredActions(dpy, wks_1);

        XpexUpdateWorkstation(dpy, wks_1);

    } /* end-for */
}

/*
 *
 *
 *
 */

static void
load_box()
{
    XpexCoord3D p[5];
    XpexVector3D xlv;
    XpexMatrix mx;
    XpexVector3D scale;
    XpexVector3D shift;
    int err = 0;
    float xang, yang, zang;

    XpexListOfCoord3D side;

    side.num_points = 4; side.points = p;

    setup(4, mx);

	Print((stderr, "load_box(); IN\n"));

	cube = XpexCreateStructure(dpy);

	Print((stderr, "create structure for cube:\n"));
	Print((stderr, "print_structure_info:cube: %u\n", cube));
	print_structure_info(dpy, cube);

	XpexStoreOutputCmds(dpy, cube);
		XpexLabel(dpy, STYLE);
		XpexSetInteriorStyle(dpy, XpexInteriorStyle_Solid);
		XpexSetHlhsrId(dpy, Xpex_Off); 
/*
 * ... make a square to make a front and back of the cube
 */
    p[0].x =  5.; p[0].y =  5.; p[0].z = 5.;
    p[1].x = 15.; p[1].y =  5.; p[1].z = 5.;
    p[2].x = 15.; p[2].y = 15.; p[2].z = 5.;
    p[3].x =  5.; p[3].y = 15.; p[3].z = 5.;
                   
/* front */
    xlv.x = 0.0; xlv.y = 0.0; xlv.z = 0.0;
    XpexTranslate(&xlv, mx);

	/* XpexStoreOutputCmds(dpy, cube); */
	XpexSetLocalTransform(dpy, Xpex_Replace, mx);

#ifdef AUTO_BOX_DEBUG
    print_matrix( mx);
#endif

	/* XpexStoreOutputCmds(dpy, cube); */
	XpexSetSurfaceColorIndex(dpy, RED);

	FILL_AREA_CALL(dpy, p, 4);

/* back */

    xlv.x = 0.0; xlv.y = 0.0; xlv.z = 10.0;

    XpexTranslate(&xlv, mx);

	/* XpexStoreOutputCmds(dpy, cube); */
	XpexSetLocalTransform(dpy, Xpex_Replace, mx);

#ifdef AUTO_BOX_DEBUG
    print_matrix(mx);
#endif /* AUTO_BOX_DEBUG */

    XpexSetSurfaceColorIndex(dpy, CYAN);

	FILL_AREA_CALL(dpy, p, 4);

/* right */

    xlv.x   = 5.0; xlv.y   =      5.0; xlv.z   = 5.0;
    scale.x = 1.0; scale.y =      1.0; scale.z = 1.0;         
    xang    = 0.0; yang    = 3.14*1.5; zang    = 0.0; 
    shift.x = 0.0; shift.y =      0.0; shift.z = 0.0;

	XpexBuildTransformationMatrix(&xlv, &shift, 
	xang, yang, zang, &scale, mx);

	/* XpexStoreOutputCmds(dpy, cube); */
	XpexSetLocalTransform(dpy, Xpex_Replace, mx);

	/* XpexStoreOutputCmds(dpy, cube); */
    XpexSetSurfaceColorIndex(dpy, GREEN);

	FILL_AREA_CALL(dpy, p, 4);

/* left */

    xlv.x   = 15.0; xlv.y   =       5.0; xlv.z   = 5.0;
    scale.x =  1.0; scale.y =       1.0; scale.z = 1.0;         
    xang    =  0.0; yang    = -3.14*1.5; zang    = 0.0; 
    shift.x =  0.0; shift.y =       0.0; shift.z = 0.0;

    XpexBuildTransformationMatrix(&xlv, &shift, 
	xang, yang, zang, &scale, mx);

	/* XpexStoreOutputCmds(dpy, cube); */
	XpexSetLocalTransform(dpy, Xpex_Replace, mx);

	XpexSetSurfaceColorIndex(dpy, BLUE);

	FILL_AREA_CALL(dpy, p, 4);

/* bottom */

    xlv.x   =         5.0; xlv.y   = 5.0; xlv.z   = 5.0;
    scale.x =         1.0; scale.y = 1.0; scale.z = 1.0;         
    xang    = -1.5 * 3.14; yang    = 0.0; zang    = 0.0; 
    shift.x =         0.0; shift.y = 0.0; shift.z = 0.0;

	XpexBuildTransformationMatrix(&xlv, &shift, 
	xang, yang, zang, &scale, mx);

	/* XpexStoreOutputCmds(dpy, cube); */
    	XpexSetLocalTransform(dpy, Xpex_Replace, mx);
    	XpexSetSurfaceColorIndex(dpy, YELLOW);
		FILL_AREA_CALL(dpy, p, 4);
/* top */

    xlv.x   = 5.0;        xlv.y   = 15.0; xlv.z   = 5.0;
    scale.x = 1.0;        scale.y =  1.0; scale.z = 1.0;         
    xang    = 1.5 * 3.14; yang    =  0.0; zang    = 0.0; 
    shift.x = 0.0;        shift.y =  0.0; shift.z = 0.0;

    XpexBuildTransformationMatrix(&xlv, &shift, 
	xang, yang, zang, &scale, mx);

	/* XpexStoreOutputCmds(dpy, cube); */
    	XpexSetLocalTransform(dpy, Xpex_Replace, mx);
    	XpexSetSurfaceColorIndex(dpy, MAGENTA);
		FILL_AREA_CALL(dpy, p, 4);

Print((stderr, "print_structure_info:cube:\n", cube));
print_structure_info(dpy, cube);
Print((stderr, "load_box(); OUT\n")); /* DEBUG STATEMENT */
}
	
XpexViewMap3D view_map = {
    { -20.0, 20.0, -20.0, 20.0},
    { 0.0, .5, 0.5, 1.0, 0.0, 1.0},
    Xpex_PERSPECTIVE,
    { 0., 0., 1000.0},
    0.0,
    -40.0,
    40.0
};

XpexViewMap3D view_map2 = {
    { -20.0, 20.0, -20.0, 20.0},
    { 0.5, 1.0, 0.5, 1.0, 0.0, 1.0},
    Xpex_PERSPECTIVE,
    { 0., 0., 1000.0},
    0.0,
    -40.0,
    40.0
};

XpexViewMap3D view_map3 = {
    { -20.0, 20.0, -20.0, 20.0},
    { 0.0, .5, 0., .5, 0.0, 1.0},
    Xpex_PERSPECTIVE,
    { 0., 0., 1000.0},
    0.0,
    -40.0,
    40.0
};

XpexViewMap3D view_map4 = {
    { -20.0, 20.0, -20.0, 20.0},
    { 0.5, 1.0, 0.,.5, 0.0, 1.0},
    Xpex_PERSPECTIVE,
    { 0., 0., 1000.0},
    0.0,
    -40.0,
    40.0
};

static void
load_views()
{
    static XpexCoord3D  vrp = {0., 0., 0 };    /*  origin */
    static XpexVector3D vpn = {1.0, 1.0, 1.0 };/*  view plane normal */
    static XpexVector3D vup = {0.0, 1.0, 0.0 };/*  view up vector */
		

    int err;
    XpexViewRep vrep;                           /*  view structure */

	Print((stderr, "load_views(); IN\n"));

    err = XpexEvaluateViewOrientationMatrix(&vrp, &vpn, &vup, 
	vrep.view.orientation);

    if (err != 0)  {
        (void) printf("view orientation error ");
	}

    err = XpexEvaluateViewMappingMatrix(&view_map, vrep.view.mapping);

    if (err != 0) {
        (void) printf("view mapping error ");
	}

    vrep.index = 1;
    vrep.view.clipFlags = 0;
    vrep.view.clipLimits.minval.x = view_map.viewport.x_min;
    vrep.view.clipLimits.minval.y = view_map.viewport.y_min;
    vrep.view.clipLimits.minval.z = view_map.viewport.z_min;
    vrep.view.clipLimits.maxval.x = view_map.viewport.x_max;
    vrep.view.clipLimits.maxval.y = view_map.viewport.y_max;
    vrep.view.clipLimits.maxval.z = view_map.viewport.z_max;
    
    XpexSetViewRep(dpy, wks_1, &vrep);
/*
 *   viewport # 2
 */
    err = XpexEvaluateViewOrientationMatrix(&vrp, &vpn, &vup, 
	vrep.view.orientation);

    if (err != 0) { 
        (void) printf("view orientation error ");
	}

    err = XpexEvaluateViewMappingMatrix(&view_map2, 
	vrep.view.mapping);

	if (err != 0) { 
		(void) printf("view mapping error ");
	}

    vrep.index = 2;
    vrep.view.clipFlags = 0;
    vrep.view.clipLimits.minval.x = view_map2.viewport.x_min;
    vrep.view.clipLimits.minval.y = view_map2.viewport.y_min;
    vrep.view.clipLimits.minval.z = view_map2.viewport.z_min;
    vrep.view.clipLimits.maxval.x = view_map2.viewport.x_max;
    vrep.view.clipLimits.maxval.y = view_map2.viewport.y_max;
    vrep.view.clipLimits.maxval.z = view_map2.viewport.z_max;

    XpexSetViewRep(dpy, wks_1, &vrep);
/*
 *   viewport # 3
 */
	err = XpexEvaluateViewOrientationMatrix(&vrp, &vpn, &vup, 
	vrep.view.orientation);

	if (err != 0)  {
		(void) printf("view orientation error ");
	}

    err = XpexEvaluateViewMappingMatrix(&view_map3, 
	vrep.view.mapping);

	if (err != 0) {
		(void) printf("view mapping error ");
	}

    vrep.index = 3;
    vrep.view.clipFlags = 0;
    vrep.view.clipLimits.minval.x = view_map3.viewport.x_min;
    vrep.view.clipLimits.minval.y = view_map3.viewport.y_min;
    vrep.view.clipLimits.minval.z = view_map3.viewport.z_min;
    vrep.view.clipLimits.maxval.x = view_map3.viewport.x_max;
    vrep.view.clipLimits.maxval.y = view_map3.viewport.y_max;
    vrep.view.clipLimits.maxval.z = view_map3.viewport.z_max;

    XpexSetViewRep(dpy, wks_1, &vrep);
/*
 *   viewport # 4
 */
    err = XpexEvaluateViewOrientationMatrix(&vrp, &vpn, &vup, 
	vrep.view.orientation);

    if (err != 0) {
		(void) printf ("view orientation error ");
	}

    err = XpexEvaluateViewMappingMatrix(&view_map4, vrep.view.mapping);

    if (err != 0)  {
        (void) printf("view mapping error ");
	}

    vrep.index = 4;
    vrep.view.clipFlags = 0;
    vrep.view.clipLimits.minval.x = view_map4.viewport.x_min;
    vrep.view.clipLimits.minval.y = view_map4.viewport.y_min;
    vrep.view.clipLimits.minval.z = view_map4.viewport.z_min;
    vrep.view.clipLimits.maxval.x = view_map4.viewport.x_max;
    vrep.view.clipLimits.maxval.y = view_map4.viewport.y_max;
    vrep.view.clipLimits.maxval.z = view_map4.viewport.z_max;

    XpexSetViewRep(dpy, wks_1, &vrep);

	Print((stderr, "load_views(); OUT\n"));
}

static void
build_css()
{
    XpexCoord2D  line[2];
    XpexMatrix   r1, r2, r3, r4;
    XpexCoord2D  text_pt;

    setup(4, r1);
    setup(4, r2);
    setup(4, r3);
    setup(4, r4);

	Print((stderr, "build_css(); IN\n"));

/*
 * ... open the root structure
 */

/*
 *    popen_struct( ROOT); 
 *
 */
	
    /* XPexSetHlhsrMode( display, wks_1, XPexHlhsrOff); */

	root = XpexCreateStructure(dpy);

    sub = XpexCreateStructure(dpy);

	Print((stderr, "print_structure_info: root\n", root));
	print_structure_info(dpy, root);

    XpexSetEditingMode(dpy, root, XpexStructure_Insert);

	SET_ELEM_PTR_TO_BEGINNING(dpy, root);
/*
 * ...  set up lines that split the viewports
 *
 */

	XpexStoreOutputCmds(dpy, root);
		XpexSetLineWidth(dpy, 3.0);
		XpexSetLineColorIndex(dpy, WHITE);

    	line[0].x = .5; line[0].y = 0.0; 
    	line[1].x = .5; line[1].y = 1.0;
		XpexPolyline2D(dpy, line, 2);

    	line[0].x = 0.0; line[0].y = .5;
    	line[1].x = 1.0; line[1].y = .5;
		XpexPolyline2D(dpy, line, 2);
/*
 * ... text to explain each viewport
 */

		XpexSetTextColorIndex(dpy, YELLOW);
		XpexSetTextFontIndex(dpy, -2);
		XpexSetCharHeight(dpy, 0.025);
		XpexLabel(dpy, QUAD_2);

    	text_pt.x = 0; text_pt.y = .52;
		XpexText2D( dpy, &text_pt, " X Rotation", 
		strlen(" X Rotation"));

		XpexLabel(dpy, QUAD_1);

    	text_pt.x = .5; text_pt.y = .58;
		XpexText2D(dpy, &text_pt, " Y Rotation", 
		strlen(" Y Rotation"));

		XpexLabel(dpy, QUAD_1_1);

		text_pt.x = .5; text_pt.y = .52;
		XpexText2D(dpy, &text_pt, " Backfaces Culled", 
		strlen(" Backfaces Culled"));

    	XpexLabel(dpy, QUAD_3);

		text_pt.x = 0; text_pt.y = .02;
		XpexText2D(dpy, &text_pt, " Z Rotation",
		strlen(" Z Rotation"));

    	XpexLabel(dpy, QUAD_4);

    	text_pt.x = .5; text_pt.y = .08;
		XpexText2D(dpy, &text_pt, " XYZ Rotation",
		strlen(" XYZ Rotation"));

		XpexLabel(dpy, QUAD_4_1);
		text_pt.y = .02;
		XpexText2D(dpy, &text_pt, " FrontFaces Culled",
		strlen(" FrontFaces Culled"));

		XpexSetLineWidth(dpy, 1.0);

		XpexExecuteStructure(dpy, sub);

		Print((stderr," print_structure: root %u\n", root));
		print_structure_info(dpy, root);

/* -------------------------------------------------------------- */

    /* mode_info.def_info.s_id = sub; */

	Print((stderr, "print_structure_info: sub: %u\n", sub));
	print_structure_info(dpy, sub);

    XpexSetEditingMode(dpy, sub, XpexStructure_Insert);

	SET_ELEM_PTR_TO_BEGINNING(dpy, sub);

	XpexStoreOutputCmds(dpy, sub);
		XpexSetViewIndex(dpy, 1);
		XpexLabel(dpy, 1);
		XpexSetLocalTransform(dpy, Xpex_PreConcatenate, r1);
		/* Cull no faces in this view */
		XpexSetCullingMode(dpy, 0);
		XpexExecuteStructure(dpy, cube);

		XpexSetViewIndex(dpy, 2);
		XpexLabel(dpy, 2);
		XpexSetLocalTransform(dpy, Xpex_PreConcatenate, r2);
		/* Cull Backfaces in this view */
		XpexSetCullingMode(dpy, 0);
		XpexExecuteStructure( dpy, cube);

		XpexSetViewIndex(dpy, 3);
    	XpexLabel(dpy, 3);
		XpexSetLocalTransform(dpy, Xpex_PreConcatenate, r3);
		/* Cull no faces in this view */
		XpexSetCullingMode(dpy, 0);
		XpexExecuteStructure(dpy, cube);

		XpexSetViewIndex(dpy, 4);
    	XpexLabel(dpy, 4);
		XpexSetLocalTransform(dpy, Xpex_PreConcatenate, r4);
		/* Cull frontfaces in this view */
		XpexSetCullingMode(dpy, 0);
    	XpexExecuteStructure(dpy, cube);

		Print((stderr, "print_structure_info: sub: %u\n", sub));
		print_structure_info(dpy, sub);

	Print((stderr, "build_css(); OUT\n"));
}

static void
edit_css()
{
#ifdef AUTO_BOX_DEBUG
Print(( stderr, "edit_css(): cube: %u\n", cube));
print_structure_info(dpy, cube);
#endif /* AUTO_BOX_DEBUG */

	/* pModeInfo->def_info.s_id = cube; */

	XpexSetEditingMode(dpy, cube, XpexStructure_Replace);

	SET_ELEM_PTR_TO_BEGINNING(dpy, cube);
	XpexSetElementPointerAtLabel(dpy, cube, STYLE, 1);

	XpexStoreOutputCmds(dpy, cube);
		XpexSetInteriorStyle(dpy, XpexInteriorStyle_Hollow);
}

static void
relabel_quadrants()
{
    XpexCoord2D     text_pt;
/*
 * ... relabel the quadrants
 */

    Print(( stderr, "relabel_quadrants();\n"));
    /* mode_info.def_info.s_id = root; */
    Print(( stderr, "print_structure_info: root\n"));
	print_structure_info( dpy, root);

	SET_ELEM_PTR_TO_BEGINNING(dpy, root);
    XpexSetEditingMode(dpy, root, XpexStructure_Replace);
    XpexSetElementPointerAtLabel(dpy, root, QUAD_2, 1);

	XpexStoreOutputCmds(dpy, root);
    	text_pt.x = 0.0; text_pt.y = 0.52;
    	XpexText2D(dpy, &text_pt, " Zooming", strlen(" Zooming"));

    XpexSetElementPointerAtLabel(dpy, root, QUAD_1, 1);

	XpexStoreOutputCmds(dpy, root);
    	text_pt.x = .5; text_pt.y = .52;
    	XpexText2D(dpy, &text_pt, " Zooming", strlen(" Zooming"));

/* ----- */

    XpexSetElementPointerAtLabel(dpy, root, QUAD_3, 1);

	XpexStoreOutputCmds(dpy, root);
    	text_pt.x = 0.0; text_pt.y = .02;
    	XpexText2D(dpy, &text_pt, " Zooming", strlen(" Zooming"));

/* ----- */

    XpexSetElementPointerAtLabel(dpy, root, QUAD_4, 1);

	XpexStoreOutputCmds(dpy, root);
    	text_pt.x = .5; text_pt.y = .02;
    	XpexText2D(dpy, &text_pt, " Zooming", strlen(" Zooming"));

/* Remove the "culling labels from the quadrants */

    Print(( stderr, "print_structure_info: root\n"));
	print_structure_info(dpy, root);
	SET_ELEM_PTR_TO_BEGINNING(dpy, root);

	XpexSetElementPointerAtLabel(dpy, root, QUAD_1_1, 1);
	DELETE_ELEM(dpy, root);

	XpexSetElementPointerAtLabel(dpy, root, QUAD_4_1, 1);
	DELETE_ELEM(dpy, root);
    Print(( stderr, "print_structure_info: root\n"));
	print_structure_info(dpy, root);

	Print((stderr, "relabel quadrants - done\n"));
}

static void
InitWks()
{
	static void create_phigs_wks();

    Print((stderr, "InitWks();\n"));
/* Initialization of matrices */

    /* setup( 4, mat);
    setup( 4, s);

    setup( 4, r1);
    setup( 4, r2);
    setup( 4, r3);
    setup( 4, r4); */
/*
 * ...  set some constants for building the matrix
 */        
    pt.x    = 10.0; pt.y    = 10.0; pt.z    = 10.0;
    shift.x =  0.0; shift.y =  0.0; shift.z =  0.0;
    scale.x =  1.0; scale.y =  1.0; scale.z =  1.0;
/*
 * ...  make identity matrices
 */
    setup( 4, bldmat );
    setup( 4, bldmat2 );
    setup( 4, bldmat3 );
    setup( 4, bldmat4 );

    /* setup_colormap(); */

/*
 * ... load the structure containing the cube
 */
    load_box(); 

    build_css();

    XSynchronize(dpy, 1);
/*
 *   ... open a workstation 
 */
    create_phigs_wks();
/*
 * ... initialize the viewports
 */
    load_views();
}

InitPEX()
{

	if (!XpexInitialize(dpy)) {
		(void) fprintf(stderr, "PEX not supported\n");
		exit(-1);
	}
	
	Print((stderr,"***** PEX initialized *****\n"));
	XpexSetFloatingPointFormat(dpy, Xpex_Ieee_754_32);
}

static void
Initialize()
{
	dpy = XOpenDisplay(NULL);

	if (!dpy) {
		(void) fprintf(stderr,"Connection Failed\n");
		exit(-1);
	}
	InitPEX();
	SetWindowLabel("PEXlib - auto box");
	SetIconLabel("PEXlib - auto box");
	drawable = CreateWindow(dpy);
	Print((stderr,"Opened Window\n"));
	InitWks();

	XpexSetupColormap(dpy, drawable, wks_res.color_approx_table); 
}

static void
draw_image()
{
	float scale_factor;

/*
 * ... post the structure  --- display it
 */

	Print((stderr, "wks_1: %d\troot: %d\n", wks_1, root));
	XpexPostStructure(dpy, wks_1, root, 0.0);
/*
 * ... rotate the four views
 */

	Print(( stderr, "rotate_views(); -- first time\n"));
	rotate_views();

/*
 * ... turn off solids and make wireframe
 */
	edit_css();
/*
 * ... rotate the four views in wireframe
 */
	Print((stderr, "rotate_views(); -- second time\n"));
	rotate_views();

	relabel_quadrants(); 
	
/*
 * ... scale down the four views
 */
	scale_factor = .5;
   	scale_views(scale_factor);
/*
 * ... scale up the four views
 */
	scale_factor = 2.0;
   	scale_views( scale_factor );

/*
 * test traversal
 */

	XpexExecuteDeferredActions(dpy, wks_1);
	XpexRedrawAllStructures(dpy, wks_1);

	Print((stderr, "draw_image\n"));
}

static void
create_phigs_wks()
{
    pWksRes->line_bundle = 
    XpexCreateLineBundleLUT(dpy, drawable);

    pWksRes->marker_bundle =
    XpexCreateMarkerBundleLUT(dpy,drawable);

    pWksRes->text_bundle =
    XpexCreateTextBundleLUT(dpy,drawable);

    pWksRes->interior_bundle = 
    XpexCreateInteriorBundleLUT(dpy,drawable);

    pWksRes->edge_bundle = 
    XpexCreateEdgeBundleLUT(dpy,drawable);

    pWksRes->pattern_table = 
    XpexCreatePatternLUT(dpy,drawable);

    pWksRes->text_font_table = 
    XpexCreateTextFontLUT(dpy,drawable);

    pWksRes->color_table = 
    XpexCreateColorLUT(dpy,drawable);

    pWksRes->light_table = 
    XpexCreateLightLUT(dpy,drawable);

    pWksRes->depth_cue_table = 
    XpexCreateDepthCueLUT(dpy,drawable);

    pWksRes->color_approx_table = 
    XpexCreateColorApproxLUT(dpy,drawable);

    pWksRes->highlight_incl = XpexCreateNameSet(dpy);
    pWksRes->highlight_excl = XpexCreateNameSet(dpy);
    pWksRes->invis_incl = XpexCreateNameSet(dpy);
    pWksRes->invis_excl = XpexCreateNameSet(dpy);

    wks_1 = XpexCreatePhigsWks(dpy, 
	drawable, pWksRes, Xpex_SingleBuffered);
}

static void
quit_program()
{
/* 	clean up and close 
 *	the workstation and Phigs
 */
 /**
	sleep(5);
	pclose_ws(WS1);
	pclose_phigs();
 **/
        Print((stderr, "quit_program\n"));
	XpexFreePhigsWks(dpy, wks_1);
}

main()
{
	Initialize();
	draw_image();
	quit_program();
	exit(0);
}
