/* $Header: beach_ball.Xpex.c,v 2.1 91/07/11 12:33:39 sinyaw Exp $ */

/*****************************************************************
Copyright 1991 by Sony Microsystems Company, San Jose, California

					All Rights Reserved

Permission to use, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and
that both that copyright notice and this permission notice appear
in supporting documentation, and that the name of Sony not be used
in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.

SONY DISCLAIMS ANY AND ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
INCLUDING ALL EXPRESS WARRANTIES AND ALL IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS, FOR A PARTICULAR PURPOSE. IN NO EVENT
SHALL SONY BE LIABLE FOR ANY DAMAGES OF ANY KIND, INCLUDING BUT NOT
LIMITED TO SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES RESULTING FROM
LOSS OF USE, DATA OR LOSS OF ANY PAST, PRESENT, OR PROSPECTIVE PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIENCE OR OTHER TORTIOUS ACTION, 
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS 
SOFTWARE.
*****************************************************************/

/* beach_ball.Xpex.c */

#include <stdio.h>
#include <errno.h>
#include <ctype.h>
#include <math.h>

#include <X11/Xlib.h>
#include <X11/Xatom.h>
#include <X11/Xutil.h>

#include "Xpexlib.h"
#include "Xpexutil.h"
#include "window.h"

static Display *dpy;
static Drawable drawable;
static XpexWksResources wks_res;

/* PEX Resource Ids */
XpexPhigsWks    wks1;
XpexStructure   sphere;
XpexStructure   locations;
XpexStructure   root;

#define FILLAREA3D   
#define MAX_SPHERES         100

/* Labels */
#define INTERIOR_STYLE      1
#define EDGE_FLAG           2

#define BLACK          0
#define WHITE          1
#define RED            2
#define GREEN          3
#define BLUE           4
#define YELLOW         5

#define WC_MIN        -5.0
#define WC_MAX         5.0

#define TIME_INC       1.0
#define VELOCITY       0.2

#ifdef SYSV
#define MAX_RAND 32767
#else
#define MAX_RAND 2147483647
#endif

#ifndef MIN
#define MIN( a, b)    (((a) < (b)) ? (a) : (b))
#endif
#ifndef MAX
#define MAX( a, b)    (((a) > (b)) ? (a) : (b))
#endif

/* Sphere velocity and location data. */
typedef struct {
    XpexVector3D  velocity;
    XpexMatrix    position;
} Sphere_data;

static XpexMatrix identity = { 
	1.0, 0.0, 0.0, 0.0,
   	0.0, 1.0, 0.0, 0.0,
   	0.0, 0.0, 1.0, 0.0,
   	0.0, 0.0, 0.0, 1.0 
};

static Sphere_data    sphere_data[MAX_SPHERES];
static int            sphere_count = 0;
static int				num_spheres = 1;

/* Radius of spheres. */
static double         radius = 1.0;

/* Viewing parameters */
static XpexViewRep  rep;
static XpexViewMap3D  map;
static XpexCoord3D   vrp = { 0.0, 0.0, 0.0 };
static XpexVector3D  vup = { 0.0, 0.0, 1.0 };
static XpexVector3D  vpn = { .680, -1.0, .260 };

static void
random_velocity(v)
XpexVector3D *v;
{
    extern int  random();

    int         Xr, Yr;
    float   Xv, Yv, Zv, XYv, V2;

    Xv = Yv = Zv = 0.0;
    V2 = VELOCITY * VELOCITY;

    Xr = random();
    Xv = ((float)Xr/(float)MAX_RAND) * (VELOCITY/sqrt(3.0));
    if ( Xr % 2 )
    Xv = - Xv;

    Yr = random();
    Yv = ((float)Yr/(float)MAX_RAND) * sqrt(V2 - Xv * Xv);
    if ( Yr % 2 )
    Yv = - Yv;

    XYv = sqrt(Xv * Xv + Yv * Yv);
    if ( XYv < VELOCITY ) {
    Zv = sqrt(V2 - XYv * XYv);
    if ( random() % 2 )
        Zv = - Zv;
    }

    v->x = Xv;
    v->y = Yv;
    v->z = Zv;
}

static void
add_sphere()
{
    register Sphere_data *data;

    if (sphere_count < MAX_SPHERES) {
    	data = &sphere_data[sphere_count++];
    	memcpy(data->position, identity, sizeof(XpexMatrix));
    	random_velocity(&data->velocity);

    	XpexSetEditingMode(dpy, locations, XpexStructure_Insert);

		XpexStoreOutputCmds(dpy, locations);
    		XpexSetLocalTransform(dpy,Xpex_Replace, data->position);
    		XpexExecuteStructure(dpy, sphere);
    }
}

static void
init_view_mapping()
{
    map.proj_type = Xpex_PARALLEL;
    map.proj_ref_point.z = 6.0 * WC_MAX;
    map.window.x_min = WC_MIN; map.window.x_max = WC_MAX;
    map.window.y_min = WC_MIN; map.window.y_max = WC_MAX;
    map.front_plane = 2.0 * WC_MAX;
    map.back_plane =  2.0 * WC_MIN;
    map.view_plane =  0.4 * map.proj_ref_point.z;
    map.proj_ref_point.x = (map.window.x_min + map.window.x_max) / 2.0;
    map.proj_ref_point.y = (map.window.y_min + map.window.y_max) / 2.0;
    map.viewport.x_min = 0.0; map.viewport.x_max = 1.0;
    map.viewport.y_min = 0.0; map.viewport.y_max = 1.0;
    map.viewport.z_min = 0.0; map.viewport.z_max = 1.0;
}

static void
eval_view_rep(rep)
	XpexViewRep  *rep;
{
    if (XpexEvaluateViewMappingMatrix(&map, rep->view.mapping)) {
		(void) fprintf(stderr, "Error from eval mapping\n");
	}

    if (XpexEvaluateViewOrientationMatrix(&vrp, &vpn, &vup, 
		rep->view.orientation)) {
        (void) fprintf(stderr, "Error from eval orientation\n");
	}
}

static void
set_view_rep()
{
    eval_view_rep(&rep);
    rep.index = 1;
    rep.view.clipFlags = Xpex_ClipXY | Xpex_ClipBack | Xpex_ClipFront;
    rep.view.clipLimits.minval.x = map.viewport.x_min;
    rep.view.clipLimits.minval.y = map.viewport.y_min;
    rep.view.clipLimits.minval.z = map.viewport.z_min;
    rep.view.clipLimits.maxval.x = map.viewport.x_max;
    rep.view.clipLimits.maxval.y = map.viewport.y_max;
    rep.view.clipLimits.maxval.z = map.viewport.z_max;

    XpexSetViewRep(dpy, wks1, &rep);
}

static void
build_sphere()
{
    extern int facet_sphere();
    
    XpexListOfCoord3D *facets;
    int            num_facets, num_lat = 6, num_long = 8;
    register int   i;

    num_facets = facet_sphere(radius, num_lat, num_long, &facets);
    (void) fprintf(stderr, "numfacets = %d \n", num_facets);
    
    for (i = 0; i < num_facets; i++) {
		XpexStoreOutputCmds(dpy, sphere);
        XpexSetSurfaceColorIndex(dpy, (i % 6) + 2);
#ifdef FILLAREA3D
        XpexFillArea(dpy, Xpex_UnknownShape, True, 
			facets[i].points, facets[i].num_points);
#else /* !FILLAREA3D */
        XpexFillAreaSet(dpy, Xpex_UnknownShape, 
			True, Xpex_UnknownContour, &facets[i], 1);
#endif /* FILLAREA3D */
    }
    free(facets); 
}

static void
build_css(edge_flag, int_style)
	int	edge_flag;
	int	int_style;
{
    XpexMatrix mat, mat2, newmat;

    XpexRotateY(3.14/2, mat2);
    XpexRotateX(3.14/2, mat);
    XpexComposeMatrix(mat, mat2, newmat);

    XpexSetEditingMode(dpy, root, XpexStructure_Insert);

	XpexStoreOutputCmds(dpy, root);
    	XpexSetViewIndex(dpy, 1);
    	XpexSetHlhsrId(dpy, 1); 
    	XpexSetSurfaceColorIndex(dpy, YELLOW);
    	XpexSetSurfaceEdgeColorIndex(dpy, GREEN);
    	XpexLabel(dpy, EDGE_FLAG);
    	XpexSetSurfaceEdgeFlag(dpy, edge_flag);
    	XpexLabel(dpy, INTERIOR_STYLE);
    	XpexSetInteriorStyle(dpy, int_style);
    	XpexExecuteStructure(dpy, locations);
		XpexSetLocalTransform(dpy, Xpex_Replace, newmat); 

    build_sphere();
}

#define NEW_RI( _v, _dt, _r ) \
    { \
    (_r) += (_v) * (_dt); \
    if ( (_r) >= WC_MAX ) { \
         (_r) = 2.0 * WC_MAX - (_r); \
         (_v) = -(_v); \
    } else if ( (_r) <= WC_MIN ) { \
         (_r) = 2.0 * WC_MIN - (_r); \
         (_v) = -(_v); \
    } \
    }

static void
move_spheres()
{
    register Sphere_data     *data;
    register int             i;    
             XpexElementPos  elem_pos;

    XpexSetEditingMode(dpy, locations, XpexStructure_Replace);

    elem_pos.whence = Xpex_Beginning;
    elem_pos.offset = 1;
    XpexSetElementPointer(dpy, locations, &elem_pos);

    for (i = 0; i < sphere_count; i++) {
    	data = &sphere_data[i];
		NEW_RI(data->velocity.x, TIME_INC, data->position[0][3])
		NEW_RI(data->velocity.y, TIME_INC, data->position[1][3])
		NEW_RI(data->velocity.z, TIME_INC, data->position[2][3])
		XpexStoreOutputCmds(dpy, locations);
			XpexSetLocalTransform(dpy, Xpex_Replace, data->position);
		elem_pos.whence = Xpex_Current;
		elem_pos.offset = 2;
        XpexSetElementPointer(dpy, locations, &elem_pos);
    } /* end-for */
}

static void
InitWks()
{
	static void create_phigs_wks();

    int  hidden_surf = XpexHlhsr_Off;

/*
 *   ... open a workstation
 */

    create_phigs_wks();

    XpexSetupColormap(dpy, drawable, wks_res.color_approx_table); 

    init_view_mapping();
    XpexSetDisplayUpdateMode(dpy, wks1, Xpex_VisualizeNone);
    XpexSetHlhsrMode(dpy, wks1, hidden_surf);
    set_view_rep();

	(void) fprintf(stderr, "wks1: %u, root: %u\n", wks1, root);
    XpexPostStructure(dpy, wks1, root, 1.0);
    XpexSetDisplayUpdateMode(dpy, wks1, Xpex_VisualizeEach);
}

static void
Initialize()
{
	InitX();
	InitPEX();
	InitStructures();
	SetWindowLabel("beach_ball");
	SetIconLabel("beach_ball");
	drawable = CreateWindow(dpy);
	XSynchronize(dpy, 1); 
	InitWks();
}

InitX()
{
	dpy = XOpenDisplay(NULL);

	if (!dpy) {
		(void) fprintf(stderr, "Connection Failed\n");
		exit(-1);
	}
}

InitPEX()
{
	if (!XpexInitialize(dpy)) {
		(void) fprintf(stderr,"PEX not supported\n");
		exit(-1);
	}
	XpexSetFloatingPointFormat(dpy, Xpex_Ieee_754_32);
}

InitStructures()
{
    int  edge_flag = Xpex_On;
    int  int_style = XpexInteriorStyle_Solid;

    sphere = XpexCreateStructure(dpy);
    locations = XpexCreateStructure(dpy);
    root = XpexCreateStructure(dpy);

    build_css(edge_flag, int_style);

	while (num_spheres--) {
		add_sphere();
	}
	XFlush(dpy);
}

static void
DrawImage()
{
    clock_reset();

    for (;;) {
    	XpexSetDisplayUpdateMode(dpy, wks1, Xpex_VisualizeNone);
        move_spheres();
        XpexSetDisplayUpdateMode(dpy, wks1, Xpex_VisualizeEach);
    }
}

static void
create_phigs_wks()
{
	wks_res.line_bundle =
	XpexCreateLineBundleLUT(dpy, drawable);

	wks_res.marker_bundle =
	XpexCreateMarkerBundleLUT(dpy,drawable);

	wks_res.text_bundle =
	XpexCreateTextBundleLUT(dpy,drawable);

	wks_res.interior_bundle =
	XpexCreateInteriorBundleLUT(dpy,drawable);

	wks_res.edge_bundle =
	XpexCreateEdgeBundleLUT(dpy,drawable);

	wks_res.pattern_table =
	XpexCreatePatternLUT(dpy,drawable);

	wks_res.text_font_table =
	XpexCreateTextFontLUT(dpy,drawable);

	wks_res.color_table =
	XpexCreateColorLUT(dpy,drawable);

	wks_res.light_table =
	XpexCreateLightLUT(dpy,drawable);

	wks_res.depth_cue_table =
	XpexCreateDepthCueLUT(dpy,drawable);

	wks_res.color_approx_table =
	XpexCreateColorApproxLUT(dpy,drawable);

	wks_res.highlight_incl = XpexCreateNameSet(dpy);
	wks_res.highlight_excl = XpexCreateNameSet(dpy);
	wks_res.invis_incl = XpexCreateNameSet(dpy);
	wks_res.invis_excl = XpexCreateNameSet(dpy);

	wks1 = XpexCreatePhigsWks(dpy,
	drawable, &wks_res, Xpex_SingleBuffered);
}

static void
handle_args( argc, argv)
	int argc;
	char *argv[];
{
	if (argc > 1) 
		num_spheres = atoi(argv[1]);

	if (!num_spheres) {
		(void) printf("%s: can't display 0 beach_balls\n",
		argv[0], num_spheres);
		exit(1);
	}
}

main( argc, argv)
	int argc;
	char *argv[];
{
#ifdef APPL_DEBUG
	(void) fprintf(stderr,"handle args\n");
#endif /* APPL_DEBUG */

	handle_args(argc, argv);

#ifdef APPL_DEBUG
	(void) fprintf(stderr,"handled args\n");
#endif /* APPL_DEBUG */

	Initialize();

	DrawImage();
}

