/* $Header: XpexPC.c,v 2.1 91/07/11 12:35:38 sinyaw Exp $ */

/*****************************************************************
Copyright 1991 by Sony Microsystems Company, San Jose, California

					All Rights Reserved

Permission to use, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and
that both that copyright notice and this permission notice appear
in supporting documentation, and that the name of Sony not be used
in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.

SONY DISCLAIMS ANY AND ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
INCLUDING ALL EXPRESS WARRANTIES AND ALL IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS, FOR A PARTICULAR PURPOSE. IN NO EVENT
SHALL SONY BE LIABLE FOR ANY DAMAGES OF ANY KIND, INCLUDING BUT NOT
LIMITED TO SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES RESULTING FROM
LOSS OF USE, DATA OR LOSS OF ANY PAST, PRESENT, OR PROSPECTIVE PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIENCE OR OTHER TORTIOUS ACTION, 
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS 
SOFTWARE.
*****************************************************************/

#include <stdio.h>
#include "Xpexlibint.h"
#include "Xpexlib.h"
#include <X11/extensions/Xext.h>
#include "extutil.h"
#include "Xpextutil.h"

#define XpexPackCARD8(_val,_list) \
{ \
CARD8 *p = (CARD8 *) (_list); \
*p = (CARD8) (_val); \
_list++; \
}

#define XpexPackCARD16(_val,_list) \
{ \
CARD16 *p = (CARD16 *) (_list); \
*p = (CARD16) (_val); \
_list++; \
}

#define XpexPackCARD32(_val,_list) \
{ \
CARD32 *p = (CARD32 *) (_list); \
*p = (CARD32) (_val); \
_list++; \
}

#define XpexPackINT16(_val,_list) \
{ \
INT16 *p = (INT16 *) (_list); \
*p = (INT16) (_val); \
_list++; \
}

#define XpexPackFLOAT(_val,_list) \
{ \
FLOAT *p = (FLOAT *) (_list); \
*p = (FLOAT) (_val); \
_list++; \
}

#define XpexPackCoord2D(_val,_list) \
{ \
pexCoord2D *p = (pexCoord2D *) (_list); \
p->x = (FLOAT) ((_val).x); \
p->y = (FLOAT) ((_val).y); \
(_list) += sizeof(pexCoord2D); \
}

#define XpexPackCoord3D(_val,_list) \
{ \
pexCoord3D *p = (pexCoord3D *) (_list); \
p->x = (FLOAT) ((_val).x); \
p->y = (FLOAT) ((_val).y); \
p->z = (FLOAT) ((_val).z); \
(_list) += sizeof(pexCoord3D); \
}

#define XpexPackTextAlignmentData(_val,_list) \
{ \
pexTextAlignmentData *p = (pexTextAlignmentData *) (_list); \
p->vertical = (CARD16) (_val.vertical); \
p->horizontal = (CARD16) (_val.horizontal); \
(_list) += sizeof(pexTextAlignmentData); \
}

#define XpexPackVector2D(_val,_list) \
{ \
pexVector2D *p = (pexVector2D *) (_list); \
p->x = (FLOAT) ((_val).x); \
p->y = (FLOAT) ((_val).y); \
(_list) += sizeof(pexVector2D); \
}

#define XpexPackVector3D(_val,_list) \
{ \
pexVector3D *p = (pexVector3D *) (_list); \
p->x = (FLOAT) ((_val).x); \
p->y = (FLOAT) ((_val).y); \
p->z = (FLOAT) ((_val).z); \
(_list) += sizeof(pexVector3D); \
}

#define XpexPackTextAlignmentData(_val,_list) \
{ \
pexTextAlignmentData *p = (pexTextAlignmentData *) (_list); \
p->vertical = (CARD16) (_val.vertical); \
p->horizontal = (CARD16) (_val.horizontal); \
(_list) += sizeof(pexTextAlignmentData); \
}

#define XpexPackColorSpecifier(_cs,_list) \
{\
int bytes; \
pexColourSpecifier *p = (pexColourSpecifier *) (_list); \
p->colourType = (pexColourType) ((_cs).color_type); \
(_list) += sizeof(pexColourSpecifier); \
Xpex_pack_color(p->colourType, &((_cs).value), (_list), &bytes); \
(_list) += bytes; \
}

#define XpexPackReflectionAttr(_ra,_list) \
{ \
int bytes; \
pexReflectionAttr *p = (pexReflectionAttr *) (_list); \
p->ambient = (FLOAT)((_ra).ambient); \
p->diffuse = (FLOAT)((_ra).diffuse); \
p->specular = (FLOAT)((_ra).specular); \
p->specularConc = (FLOAT)((_ra).specular_conc); \
p->specularColour.colourType = \
(pexColourType) ((_ra).specular_color.color_type); \
(_list) += sizeof(pexReflectionAttr); \
Xpex_pack_color((_ra).specular_color.color_type, \
(_ra).specular_color.value,list,&bytes); \
(_list) += bytes; \
}

#define Xpex_size_of_color(_t) \
( (_t == PEXIndexedColour) || (_t == PEXRgb8Colour) ? 4 : \
(_t == PEXRgb16Colour) ? 8 : 3 * sizeof(FLOAT))

#define Xpex_size_of_color_specifier(_t) \
sizeof(pexColourSpecifier) + Xpex_size_of_color(_t)

extern void Xpex_pack_color();
extern void Xpex_unpack_color();

static int size_of_pc_values();
static void pack_pc_values();
static void unpack_pc_values();

int size_of_psc_data();

XpexPC
XpexCreatePC(dpy, valuemask, values)
	Display *dpy;
	XpexPCBitmask valuemask;
	XpexPCValues *values;
{
	pexPC	pc_id;
	register XExtDisplayInfo *i = XpexFindDisplay(dpy);

	XpexCheckExtension(dpy,i,0);
	LockDisplay(dpy);
	{
		register pexCreatePipelineContextReq *req;
		int extra = size_of_pc_values(valuemask, values);

		XpexGetReqExtra(CreatePipelineContext, dpy, i, extra, req);
		req->fpFormat = XpexGetFloatingPointFormat(dpy);
		req->pc = pc_id = XAllocID(dpy);
		req->itemMask[0] = valuemask[0];
		req->itemMask[1] = valuemask[1];
		req->itemMask[2] = valuemask[2];
		if (extra) {
    		pack_pc_values(valuemask, values, (CARD32 *)(req + 1));
		}
	}
	UnlockDisplay(dpy);
	SyncHandle();

	return pc_id;
}

void
XpexCopyPC(dpy, src, dst, valuemask)
	Display  *dpy;
	XpexPC  src, dst;
	XpexPCBitmask valuemask;
{
	XExtDisplayInfo *i = XpexFindDisplay(dpy);
	XpexSimpleCheckExtension(dpy,i);

	LockDisplay(dpy);
	{
		register pexCopyPipelineContextReq *req;

		XpexGetReq(CopyPipelineContext, dpy, i, req);
		req->src = src;
		req->dst = dst;
		req->itemMask[0] = valuemask[0];
		req->itemMask[1] = valuemask[1];
		req->itemMask[2] = valuemask[2];
	}
	UnlockDisplay(dpy);
	SyncHandle();
}

void
XpexFreePC(dpy, pc_id)
	Display	*dpy;
	XpexPC	pc_id;
{	
	register XExtDisplayInfo *i = XpexFindDisplay(dpy);
	XpexSimpleCheckExtension(dpy,i);

	LockDisplay(dpy);
	{
		register pexFreePipelineContextReq *req;

		XpexGetResReq(FreePipelineContext, dpy, i, pc_id, req);
	}
	UnlockDisplay(dpy);
	SyncHandle();
}
 
Status 
XpexGetPC(dpy, pc, valueMask, values) 
	Display	*dpy; 
	XpexPC	pc; 
	XpexPCBitmask valueMask;
	XpexPCValues *values; /* RETURN */ 
{
	CARD32 *list;

	if(!_XpexGetPC(dpy, pc, valueMask, &list)) {
		return 0;
	}

	unpack_pc_values(list, valueMask, values);

	if (list) {
		XFree(list);
	}
	return 1;
}

static Status 
_XpexGetPC(dpy, pc, valueMask, list) 
	Display	*dpy; 
	XpexPC	pc; 
	XpexPCBitmask valueMask;
	CARD32 **list; /* RETURN */ 
{
	register	pexGetPipelineContextReq	*req;
				pexGetPipelineContextReply	reply;
	register	XExtDisplayInfo *i = XpexFindDisplay(dpy);

	Status 		status = 0;
	unsigned 	size;

	XpexCheckExtension(dpy,i,0);
	LockDisplay(dpy);
	XpexGetReq(GetPipelineContext, dpy, i, req);
	req->fpFormat = (pexEnumTypeIndex) XpexGetFloatingPointFormat(dpy);
	req->pc = pc;
	req->itemMask[0] = valueMask[0];
	req->itemMask[1] = valueMask[1];
	req->itemMask[2] = valueMask[2];
	if (!_XReply(dpy, (xReply *)&reply, 0, xFalse)) {
		UnlockDisplay(dpy);
		SyncHandle();
		return 0;
	}
	if ((size = reply.length * sizeof(CARD32)) > 0) {
		if (*list = (CARD32 *)Xmalloc(size)) {
			_XRead(dpy, *list, (long)size);
		} else {
			status = 0;
			_XpexClearReply(dpy, reply.length);
		}
	}
	UnlockDisplay(dpy);
	SyncHandle();

	return status;
}

void
XpexChangePC(dpy, pc, valueMask, values)
	Display  *dpy;
	XpexPC  pc;
	XpexPCBitmask valueMask;
	XpexPCValues  *values; 
{
	register	pexChangePipelineContextReq *req;
	register	XExtDisplayInfo *i = XpexFindDisplay(dpy);
	int extra = size_of_pc_values(valueMask, values);

	XpexSimpleCheckExtension(dpy,i);
	LockDisplay(dpy);
	XpexGetReqExtra(ChangePipelineContext, dpy, i, extra, req);
	req->fpFormat = (pexEnumTypeIndex) XpexGetFloatingPointFormat(dpy);
	req->pc = pc;
	req->itemMask[0] = valueMask[0];
	req->itemMask[1] = valueMask[1];
	req->itemMask[2] = valueMask[2];
    pack_pc_values(valueMask, values, (CARD32 *)(req + 1));
	UnlockDisplay(dpy);
	SyncHandle();
}

int
size_of_psc_data(p)
	register XpexParaSurfChar *p;
{
	switch (p->characteristics) {
	case PEXPSCNone:
	case PEXPSCImpDep:
		return 0;
	case PEXPSCIsoCurves:
		return sizeof(pexPSC_IsoparametricCurves);
	case PEXPSCMcLevelCurves:
		return (sizeof(pexPSC_LevelCurves) +
		p->record.mc_curves.numberIntersections * 
		sizeof(pexCoord3D));
	case PEXPSCWcLevelCurves:
		return (sizeof(pexPSC_LevelCurves) +
		p->record.wc_curves.numberIntersections * 
		sizeof(pexCoord3D));
	}
	return 0;
}

static int
size_of_pc_values(valueMask, values)
	XpexPCBitmask valueMask;
	XpexPCValues *values;
{
	int size = 0;

	if (valueMask[0] & XpexPC_MarkerType) {
		size += sizeof(CARD32);
	}
	if (valueMask[0] & XpexPC_MarkerScale) {
		size += sizeof(FLOAT);
	}
	if (valueMask[0] & XpexPC_MarkerColor) {
		size += sizeof(pexColourSpecifier) +
		Xpex_size_of_color(values->marker_color.color_type);
	}
	if (valueMask[0] & XpexPC_MarkerBundleIndex) {
		size += sizeof(CARD32);
	}
	if (valueMask[0] & XpexPC_TextFont) {
		size += sizeof(CARD32);
	}
	if (valueMask[0] & XpexPC_TextPrecision) {
		size += sizeof(CARD32);
	}
	if (valueMask[0] & XpexPC_CharExpansion) {
		size += sizeof(FLOAT);
	}
	if (valueMask[0] & XpexPC_CharSpacing) {
		size += sizeof(FLOAT);
	}
	if (valueMask[0] & XpexPC_TextColor) {
		size += sizeof(pexColourSpecifier) +
		Xpex_size_of_color(values->text_color.color_type);
	}
	if (valueMask[0] & XpexPC_CharHeight) {
		size += sizeof(FLOAT);
	}
	if (valueMask[0] & XpexPC_CharUpVector) {
		size += sizeof(pexVector2D);
	}
	if (valueMask[0] & XpexPC_TextPath) {
		size += sizeof(CARD32);
	}
	if (valueMask[0] & XpexPC_TextAlignment) {
		size += sizeof(pexTextAlignmentData);
	}
	if (valueMask[0] & XpexPC_AtextHeight) {
		size += sizeof(FLOAT);
	}
	if (valueMask[0] & XpexPC_AtextUpVector) {
		size += sizeof(pexVector2D);
	}
	if (valueMask[0] & XpexPC_AtextPath) {
		size += sizeof(CARD32);
	}
	if (valueMask[0] & XpexPC_AtextAlignment) {
		size += sizeof(pexTextAlignmentData);
	}
	if (valueMask[0] & XpexPC_AtextStyle) {
		size += sizeof(CARD32);
	}
	if (valueMask[0] & XpexPC_TextBundleIndex) {
		size += sizeof(CARD32);
	}
	if (valueMask[0] & XpexPC_LineType) {
		size += sizeof(CARD32);
	}
	if (valueMask[0] & XpexPC_LineWidth) {
		size += sizeof(FLOAT);
	}
	if (valueMask[0] & XpexPC_LineColor) {
		size += sizeof(pexColourSpecifier) +
		Xpex_size_of_color(values->line_color.color_type);
	}
	if (valueMask[0] & XpexPC_CurveApproximation) {
		size += sizeof(pexCurveApprox);
	}
	if (valueMask[0] & XpexPC_PolylineInterp) {
		size += sizeof(CARD32);
	}
	if (valueMask[0] & XpexPC_LineBundleIndex) {
		size += sizeof(CARD32);
	}
	if (valueMask[0] & XpexPC_InteriorStyle) {
		size += sizeof(CARD32);
	}
	if (valueMask[0] & XpexPC_SurfaceColor) {
		size += sizeof(pexColourSpecifier) +
		Xpex_size_of_color(values->surface_color.color_type);
	}
	if (valueMask[0] & XpexPC_SurfaceReflAttr) {
		size += sizeof(pexReflectionAttr) +
		Xpex_size_of_color(values->reflection_attr.specular_color.color_type);
	}
	if (valueMask[0] & XpexPC_SurfaceReflModel) {
		size += sizeof(CARD32);
	}
	if (valueMask[0] & XpexPC_SurfaceInterp) {
		size += sizeof(CARD32);
	}
	if (valueMask[0] & XpexPC_BfInteriorStyle) {
		size += sizeof(CARD32);
	}
	/* check second bitmask */

	if (valueMask[1] & XpexPC_BfInteriorStyleIndex) {
		size += sizeof(CARD32);
	}
	if (valueMask[1] & XpexPC_BfSurfaceColor) {
		size += sizeof(pexColourSpecifier) +
		Xpex_size_of_color(values->bf_surface_color.color_type);
	}
	if (valueMask[1] & XpexPC_BfSurfaceReflAttr) {
		size += sizeof(pexReflectionAttr) +
		Xpex_size_of_color(values->bf_reflection_attr.specular_color.color_type);
	}
	if (valueMask[1] & XpexPC_BfSurfaceReflModel) {
		size += sizeof(CARD32);
	}
	if (valueMask[1] & XpexPC_BfSurfaceInterp) {
		size += sizeof(CARD32);
	}
	if (valueMask[1] & XpexPC_SurfaceApproximation) {
		size += sizeof(pexSurfaceApprox);
	}
	if (valueMask[1] & XpexPC_CullingMode) {
		size += sizeof(CARD32);
	}
	if (valueMask[1] & XpexPC_DistinguishFlag) {
		size += sizeof(CARD32);
	}
	if (valueMask[1] & XpexPC_PatternSize) {
		size += sizeof(pexCoord2D);
	}
	if (valueMask[1] & XpexPC_PatternRefPt) {
		size += sizeof(pexCoord3D);
	}
	if (valueMask[1] & XpexPC_PatternRefVec1) {
		size += sizeof(pexVector3D);
	}
	if (valueMask[1] & XpexPC_PatternRefVec2) {
		size += sizeof(pexVector3D);
	}
	if (valueMask[1] & XpexPC_InteriorBundleIndex) {
		size += sizeof(CARD32);
	}
	if (valueMask[1] & XpexPC_SurfaceEdgeFlag) {
		size += sizeof(CARD32);
	}
	if (valueMask[1] & XpexPC_SurfaceEdgeType) {
		size += sizeof(CARD32);
	}
	if (valueMask[1] & XpexPC_SurfaceEdgeWidth) {
		size += sizeof(FLOAT);
	}
	if (valueMask[1] & XpexPC_SurfaceEdgeColor) {
		size += sizeof(pexColourSpecifier) +
		Xpex_size_of_color(values->surface_edge_color.color_type);
	}
	if (valueMask[1] & XpexPC_EdgeBundleIndex) {
		size += sizeof(CARD32);
	}
	if (valueMask[1] & XpexPC_LocalTransform) {
		size += sizeof(pexMatrix);
	}
	if (valueMask[1] & XpexPC_GlobalTransform) {
		size += sizeof(pexMatrix);
	}
	if (valueMask[1] & XpexPC_ModelClip) {
		size += sizeof(CARD32);
	}
	if (valueMask[1] & XpexPC_ModelClipVolume) {
		size += sizeof(CARD32) + 
		values->model_clip_volume.num_half_spaces * sizeof(pexHalfSpace);
	}
	if (valueMask[1] & XpexPC_ViewIndex) {
		size += sizeof(CARD32);
	}
	if (valueMask[1] & XpexPC_LightState) {
		size += sizeof(CARD32) +
		values->light_state.num_indices * sizeof(pexTableIndex);
	}
	if (valueMask[1] & XpexPC_DepthCueIndex) {
		size += sizeof(CARD32);
	}
	if (valueMask[1] & XpexPC_SetAsfValues) {
		size += sizeof(CARD32) + sizeof(CARD32);
	}
	if (valueMask[1] & XpexPC_PickId) {
		size += sizeof(CARD32);
	}
	if (valueMask[1] & XpexPC_HlhsrIdentifier) {
		size += sizeof(CARD32);
	}
	if (valueMask[1] & XpexPC_NameSet) {
		size += sizeof(CARD32);
	}
	if (valueMask[1] & XpexPC_ColorApproxIndex) {
		size += sizeof(CARD32);
	}
	if (valueMask[1] & XpexPC_RenderingColorModel) {
		size += sizeof(CARD32);
	}
	if (valueMask[1] & XpexPC_ParaSurfCharacteristics) {
		/* type + length + data */
		size += sizeof(CARD32) + 
		size_of_psc_data(&(values->psurf_char));
	}
	return size;
}

static void
pack_pc_values(valueMask, values, list)
	XpexPCBitmask valueMask;
	XpexPCValues  *values;
	CARD32 *list; /* RETURN */
{
	if (valueMask[0] & XpexPC_MarkerType) {
		XpexPackINT16(values->marker_type,list);
	}
	if (valueMask[0] & XpexPC_MarkerScale) {
		XpexPackFLOAT(values->marker_scale,list);
	}
	if (valueMask[0] & XpexPC_MarkerColor) {
		XpexPackColorSpecifier(values->marker_color,list);
	} 
	if (valueMask[0] & XpexPC_MarkerBundleIndex) {
		XpexPackCARD16(values->marker_bundle_index,list);
	}
	if (valueMask[0] & XpexPC_TextFont) {
		XpexPackCARD16(values->text_font_index,list);
	}
	if (valueMask[0] & XpexPC_TextPrecision) {
		XpexPackCARD16(values->text_precision,list);
	}
	if (valueMask[0] & XpexPC_CharExpansion) {
		XpexPackFLOAT(values->char_expansion,list);
	}
	if (valueMask[0] & XpexPC_CharSpacing) {
		XpexPackFLOAT(values->char_spacing,list);
	}
	if (valueMask[0] & XpexPC_TextColor) {
		XpexPackColorSpecifier(values->text_color,list);
	}
	if (valueMask[0] & XpexPC_CharHeight) {
		XpexPackFLOAT(values->char_height,list);
	}
	if (valueMask[0] & XpexPC_CharUpVector) {
		XpexPackVector2D(values->char_up_vector,list);
	}
	if (valueMask[0] & XpexPC_TextPath) {
		XpexPackCARD16(values->text_path,list);
	}
	if (valueMask[0] & XpexPC_TextAlignment) {
		XpexPackTextAlignmentData(values->text_alignment,list);
	}
	if (valueMask[0] & XpexPC_AtextHeight) {
		XpexPackFLOAT(values->atext_height,list);
	}
	if (valueMask[0] & XpexPC_AtextUpVector) {
		XpexPackVector2D(values->atext_up_vector,list);
	}
	if (valueMask[0] & XpexPC_AtextPath) {
		XpexPackCARD16(values->atext_path,list);
	}
	if (valueMask[0] & XpexPC_AtextAlignment) {
		XpexPackTextAlignmentData(values->atext_alignment,list);
	}
	if (valueMask[0] & XpexPC_AtextStyle) {
		XpexPackINT16(values->atext_style,list);
	}
	if (valueMask[0] & XpexPC_TextBundleIndex) {
		XpexPackCARD16(values->text_bundle_index,list);
	}
	if (valueMask[0] & XpexPC_LineType) {
		XpexPackINT16(values->line_type,list);
	}
	if (valueMask[0] & XpexPC_LineWidth) {
		XpexPackFLOAT(values->line_width,list);
	}
	if (valueMask[0] & XpexPC_LineColor) {
		XpexPackColorSpecifier(values->line_color,list);
	}
	if (valueMask[0] & XpexPC_CurveApproximation) {
		XpexPackINT16(values->curve_approx.approxMethod,list);
		XpexPackFLOAT(values->curve_approx.tolerance,list);
	}
	if (valueMask[0] & XpexPC_PolylineInterp) {
		XpexPackINT16(values->polyline_interp,list);
	}
	if (valueMask[0] & XpexPC_LineBundleIndex) {
		XpexPackCARD16(values->line_bundle_index,list);
	}
	if (valueMask[0] & XpexPC_InteriorStyle) {
		XpexPackINT16(values->interior_style,list);
	}
	if (valueMask[0] & XpexPC_InteriorStyleIndex) {
		XpexPackCARD16(values->interior_style_index,list);
	}
	if (valueMask[0] & XpexPC_SurfaceColor) {
		XpexPackColorSpecifier(values->surface_color,list);
	}
	if (valueMask[0] & XpexPC_SurfaceReflAttr) {
		XpexPackReflectionAttr(values->reflection_attr,list);
	}
	if (valueMask[0] & XpexPC_SurfaceReflModel) {
		XpexPackINT16(values->reflection_model,list);
	}
	if (valueMask[0] & XpexPC_SurfaceInterp) {
		XpexPackINT16(values->surface_interp,list);
	}
	if (valueMask[0] & XpexPC_BfInteriorStyle) {
		XpexPackINT16(values->bf_interior_style,list);
	}
	if (valueMask[1] & XpexPC_BfInteriorStyleIndex) {
		XpexPackCARD16(values->bf_interior_style_index,list);
	}
	if (valueMask[1] & XpexPC_BfSurfaceColor) {
		XpexPackColorSpecifier(values->bf_surface_color,list);
	}
	if (valueMask[1] & XpexPC_BfSurfaceReflAttr) {
		XpexPackReflectionAttr(values->bf_reflection_attr,list);
	}
	if (valueMask[1] & XpexPC_BfSurfaceReflModel) {
		XpexPackINT16(values->bf_reflection_model,list);
	}
	if (valueMask[1] & XpexPC_BfSurfaceInterp) {
		XpexPackINT16(values->bf_surface_interp,list);
	}
	if (valueMask[1] & XpexPC_SurfaceApproximation) {
		XpexPackINT16(values->surface_approx.approxMethod,list);
		XpexPackFLOAT(values->surface_approx.uTolerance,list);
		XpexPackFLOAT(values->surface_approx.vTolerance,list);
	}
	if (valueMask[1] & XpexPC_CullingMode) {
		XpexPackCARD16(values->culling_mode,list);
	}
	if (valueMask[1] & XpexPC_DistinguishFlag) {
		XpexPackCARD8(values->distinguish_flag,list);
	}
	if (valueMask[1] & XpexPC_PatternSize) {
		XpexPackCoord2D(values->pattern_size,list);
	}
	if (valueMask[1] & XpexPC_PatternRefPt) {
		XpexPackCoord3D(values->pattern_ref_pt,list);
	}
	if (valueMask[1] & XpexPC_PatternRefVec1) {
		XpexPackVector3D(values->pattern_ref_vec1,list);
	}
	if (valueMask[1] & XpexPC_PatternRefVec2) {
		XpexPackVector3D(values->pattern_ref_vec2,list);
	}
	if (valueMask[1] & XpexPC_InteriorBundleIndex) {
		XpexPackCARD16(values->interior_bundle_index,list);
	}
	if (valueMask[1] & XpexPC_SurfaceEdgeFlag) {
		XpexPackCARD16(values->surface_edge_flag,list);
	}
	if (valueMask[1] & XpexPC_SurfaceEdgeType) {
		XpexPackINT16(values->surface_edge_type,list);
	}
	if (valueMask[1] & XpexPC_SurfaceEdgeWidth) {
		XpexPackFLOAT(values->surface_edge_width,list);
	}
	if (valueMask[1] & XpexPC_SurfaceEdgeColor) {
		XpexPackColorSpecifier(values->surface_edge_color,list);
	}
	if (valueMask[1] & XpexPC_EdgeBundleIndex) {
		XpexPackCARD16(values->edge_bundle_index,list);
	}
	if (valueMask[1] & XpexPC_LocalTransform) {
		register int i, j;
		FLOAT *fval = (FLOAT *) list;
		for (j = 0; j <= 3; j++) {
			for (i = 0; i <= 3; i++) {
				*fval++ = values->local_transform[j][i];
			}
		}
		list += sizeof(pexMatrix);
	}
	if (valueMask[1] & XpexPC_GlobalTransform) {
		register int i, j;
		FLOAT *fval = (FLOAT *) list;
		for (j = 0; j <= 3; j++) {
			for (i = 0; i <= 3; i++) {
				*fval++ = values->global_transform[j][i];
			}
		}
		list += sizeof(pexMatrix);
	}
	if (valueMask[1] & XpexPC_ModelClip) {
		XpexPackCARD16(values->model_clip,list);
	}
	if (valueMask[1] & XpexPC_ModelClipVolume) {

		XpexPackCARD32(values->model_clip_volume.num_half_spaces,list);

		bcopy((char *)values->model_clip_volume.half_spaces,
		(char *)list, values->model_clip_volume.num_half_spaces *
		sizeof(pexHalfSpace));
	}
	if (valueMask[1] & XpexPC_ViewIndex) {
		XpexPackCARD16(values->view_index,list);
	}
	if (valueMask[1] & XpexPC_LightState) {
		XpexPackCARD16(values->light_state.num_indices,list);
		bcopy((char *)values->light_state.indices,(char *)list,
		values->light_state.num_indices * sizeof(pexTableIndex));
	}
	if (valueMask[1] & XpexPC_DepthCueIndex) {
		XpexPackCARD16(values->depth_cue_index,list);
	}
	if (valueMask[1] & XpexPC_SetAsfValues) {
		XpexPackCARD32(values->asf_enables,list);
		XpexPackCARD32(values->asfs,list);
	}
	if (valueMask[1] & XpexPC_PickId) {
		XpexPackCARD32(values->pick_id,list);
	}
	if (valueMask[1] & XpexPC_HlhsrIdentifier) {
		XpexPackCARD32(values->hlhsr_identifier,list);
	}
	if (valueMask[1] & XpexPC_NameSet) {
		XpexPackCARD32(values->name_set,list);
	}
	if (valueMask[1] & XpexPC_ColorApproxIndex) {
		XpexPackCARD16(values->color_approx_index,list);
	}
	if (valueMask[1] & XpexPC_RenderingColorModel) {
		XpexPackCARD16(values->rdr_color_model,list);
	}
	if (valueMask[1] & XpexPC_ParaSurfCharacteristics) {
	}
}

static void
unpack_pc_values(list, valueMask, values)
	CARD32 *list;
	XpexPCBitmask valueMask;
	XpexPCValues *values; /* RETURN */
{
	XpexColorType color_type;
	int length;

	if (valueMask[0] & XpexPC_MarkerType) {
		values->marker_type = *((pexEnumTypeIndex *) list);
		list++; 
	}
	if (valueMask[0] & XpexPC_MarkerScale) {
		values->marker_scale = *((FLOAT *) list);
		list++;
	}
	if (valueMask[0] & XpexPC_MarkerColor){
		color_type =
		values->marker_color.color_type = *((pexColourType *) list);
		list++;
		Xpex_unpack_color(color_type, list,
		&(values->marker_color.value), &length);
		list += length;
	} 
	if (valueMask[0] & XpexPC_MarkerBundleIndex) {
		values->marker_bundle_index = *((pexTableIndex *) list);
		list++;
	}
	if (valueMask[0] & XpexPC_TextFont) {
		values->text_font_index = *((pexTableIndex *) list);
		list++;
	}
	if (valueMask[0] & XpexPC_TextPrecision) {
		values->text_precision = *((CARD16 *) list);
		list++;
	}
	if (valueMask[0] & XpexPC_CharExpansion) {
		values->char_expansion = *((FLOAT *) list);
		list++;
	}
	if (valueMask[0] & XpexPC_CharSpacing) {
		values->char_spacing = *((FLOAT *) list);
		list++;
	}
	if (valueMask[0] & XpexPC_TextColor) {
		color_type =
		values->text_color.color_type = *((pexColourType *) list);
		list++;
		Xpex_unpack_color(color_type, list,
		&(values->text_color.value), &length);
		list += length;
	}
	if (valueMask[0] & XpexPC_CharHeight) {
		values->char_height = *((FLOAT *) list);
		list++;
	}
	if (valueMask[0] & XpexPC_CharUpVector) {
		values->char_up_vector.x = *((FLOAT *) list);
		list++;
		values->char_up_vector.y = *((FLOAT *) list);
		list++;
	}
	if (valueMask[0] & XpexPC_TextPath) {
		values->text_path = *((CARD16 *) list);
		list++;
	}
	if (valueMask[0] & XpexPC_TextAlignment) {
		CARD16 *s = (CARD16 *) list;
		values->text_alignment.vertical = *s++;
		values->text_alignment.horizontal = *s++;
		list = (CARD32 *) s;
	}
	if (valueMask[0] & XpexPC_AtextHeight) {
		values->atext_height = *((FLOAT *) list);
		list++;
	}
	if (valueMask[0] & XpexPC_AtextUpVector) {
		values->atext_up_vector.x = *((FLOAT *) list);
		list++;
		values->atext_up_vector.y = *((FLOAT *) list);
		list++;
	}
	if (valueMask[0] & XpexPC_AtextPath) {
		values->atext_path = *((CARD16 *) list);
		list++;
	}
	if (valueMask[0] & XpexPC_AtextAlignment) {
		CARD16 *s = (CARD16 *) list;
		values->atext_alignment.vertical =  *s++;
		values->atext_alignment.horizontal = *s++;
		list = (CARD32 *) s;
	}
	if (valueMask[0] & XpexPC_AtextStyle) {
		values->atext_style = *((INT16 *) list);
		list++;
	}
	if (valueMask[0] & XpexPC_TextBundleIndex) {
		values->text_bundle_index = *((pexTableIndex *) list);
		list++;
	}
	if (valueMask[0] & XpexPC_LineType) {
		values->line_type = *((pexEnumTypeIndex *) list);
		list++;
	}
	if (valueMask[0] & XpexPC_LineWidth) {
		values->line_type = *((FLOAT *) list);
		list++;
	}
	if (valueMask[0] & XpexPC_LineColor) {
		color_type =
		values->line_color.color_type = *((pexColourType *) list);
		list++;
		Xpex_unpack_color(color_type, list,
		&(values->line_color.value), &length);
		list += length;
	}
	if (valueMask[0] & XpexPC_CurveApproximation) {
		values->curve_approx.approxMethod = *((INT16 *) list);
		list++;
		values->curve_approx.tolerance = *((FLOAT *) list);
		list++;
	}
	if (valueMask[0] & XpexPC_PolylineInterp) {
		values->polyline_interp = *((INT16 *) list);
		list++;
	}
	if (valueMask[0] & XpexPC_LineBundleIndex) {
		values->line_bundle_index = *((CARD16 *) list);
		list++;
	}
	if (valueMask[0] & XpexPC_InteriorStyle) {
		values->interior_style = *((INT16 *) list);
		list++;
	}
	if (valueMask[0] & XpexPC_InteriorStyleIndex) {
		values->interior_style_index = *((CARD16 *) list); list++;
	}
	if (valueMask[0] & XpexPC_SurfaceColor) {
		color_type =
		values->line_color.color_type = *((pexColourType *) list);
		list++;
		Xpex_unpack_color(color_type, list,
		&(values->surface_color.value), &length);
		list += length;
	}
	if (valueMask[0] & XpexPC_SurfaceReflAttr) {
	}
	if (valueMask[0] & XpexPC_SurfaceReflModel) {
		values->reflection_model = *((INT16 *) list); list++;
	}
	if (valueMask[0] & XpexPC_SurfaceInterp) {
		values->surface_interp = *((INT16 *) list); list++;
	}
	if (valueMask[0] & XpexPC_BfInteriorStyle) {
	}
	if (valueMask[1] & XpexPC_BfInteriorStyleIndex) {
	}
	if (valueMask[1] & XpexPC_BfSurfaceColor) {
		color_type =
		values->line_color.color_type = *((pexColourType *) list);
		list++;
		Xpex_unpack_color(color_type, list,
		&(values->bf_surface_color.value), &length);
		list += length;
	}
	if (valueMask[1] & XpexPC_BfSurfaceReflAttr) {
	}
	if (valueMask[1] & XpexPC_BfSurfaceReflModel) {
		values->bf_reflection_model = *((INT16 *) list); list++;
	}
	if (valueMask[1] & XpexPC_BfSurfaceInterp) {
		values->bf_surface_interp = *((INT16 *) list); list++;
	}
	if (valueMask[1] & XpexPC_SurfaceApproximation) {
		values->surface_approx.approxMethod = *((INT16 *) list); list++;
		values->surface_approx.uTolerance = *((FLOAT *) list); list++;
		values->surface_approx.vTolerance = *((FLOAT *) list); list++;
	}
	if (valueMask[1] & XpexPC_CullingMode) {
		values->culling_mode = *((CARD16 *) list); list++;
	}
	if (valueMask[1] & XpexPC_DistinguishFlag) {
		values->distinguish_flag = *((CARD8 *) list); list++;
	}
	if (valueMask[1] & XpexPC_PatternSize) {
		values->pattern_size.x = *((FLOAT *)list); list++;
		values->pattern_size.y = *((FLOAT *)list); list++;
	}
	if (valueMask[1] & XpexPC_PatternRefPt) {
		values->pattern_ref_pt.x = *((FLOAT *)list); list++;
		values->pattern_ref_pt.y = *((FLOAT *)list); list++;
		values->pattern_ref_pt.z = *((FLOAT *)list); list++;
	}
	if (valueMask[1] & XpexPC_PatternRefVec1) {
		values->pattern_ref_vec1.x = *((FLOAT *)list); list++;
		values->pattern_ref_vec1.y = *((FLOAT *)list); list++;
		values->pattern_ref_vec1.z = *((FLOAT *)list); list++;
	}
	if (valueMask[1] & XpexPC_PatternRefVec2) {
		values->pattern_ref_vec2.x = *((FLOAT *)list); list++;
		values->pattern_ref_vec2.y = *((FLOAT *)list); list++;
		values->pattern_ref_vec2.z = *((FLOAT *)list); list++;
	}
	if (valueMask[1] & XpexPC_InteriorBundleIndex) {
		values->interior_bundle_index = *((CARD16 *) list); list++;
	}
	if (valueMask[1] & XpexPC_SurfaceEdgeFlag) {
		values->surface_edge_flag = *((CARD16 *) list); list++;
	}
	if (valueMask[1] & XpexPC_SurfaceEdgeType) {
		values->surface_edge_type = *((INT16 *) list); list++;
	}
	if (valueMask[1] & XpexPC_SurfaceEdgeWidth) {
		values->surface_edge_width = *((FLOAT *) list); list++;
	}
	if (valueMask[1] & XpexPC_SurfaceEdgeColor) {
		color_type = 
		values->surface_edge_color.color_type = *((pexColourType *) list);
		list++;
		Xpex_unpack_color(color_type, list,
		&(values->surface_edge_color.value), &length);
		list += length;
	}
	if (valueMask[1] & XpexPC_EdgeBundleIndex) {
		values->edge_bundle_index = *((CARD16 *) list);
		list++;
	}
	if (valueMask[1] & XpexPC_LocalTransform) {
	}
	if (valueMask[1] & XpexPC_GlobalTransform) {
	}
	if (valueMask[1] & XpexPC_ModelClip) {
		values->model_clip = *((CARD16 *) list);
		list++;
	}
	if (valueMask[1] & XpexPC_ModelClipVolume) {
	}
	if (valueMask[1] & XpexPC_ViewIndex) {
		values->view_index = *((CARD16 *) list);
		list++;
	}
	if (valueMask[1] & XpexPC_LightState) {
	}
	if (valueMask[1] & XpexPC_DepthCueIndex) {
		values->depth_cue_index = *((CARD16 *) list);
		list++;
	}
	if (valueMask[1] & XpexPC_SetAsfValues) {
		values->asf_enables = *list++;
		values->asfs = *list++;
	}
	if (valueMask[1] & XpexPC_PickId) {
		values->pick_id = *list++;
	}
	if (valueMask[1] & XpexPC_HlhsrIdentifier) {
		values->hlhsr_identifier = *list++;
	}
	if (valueMask[1] & XpexPC_NameSet) {
		values->name_set = *list++;
	}
	if (valueMask[1] & XpexPC_ColorApproxIndex) {
		values->color_approx_index = *((CARD16 *) list);
		list++;
	}
	if (valueMask[1] & XpexPC_RenderingColorModel) {
		values->rdr_color_model = *((INT16 *) list);
		list++;
	}
	if (valueMask[1] & XpexPC_ParaSurfCharacteristics) {
	}
}
