/* $Header: XpexRenderers.c,v 2.1 91/07/11 12:35:51 sinyaw Exp $ */


/*****************************************************************
Copyright 1991 by Sony Microsystems Company, San Jose, California

					All Rights Reserved

Permission to use, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and
that both that copyright notice and this permission notice appear
in supporting documentation, and that the name of Sony not be used
in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.

SONY DISCLAIMS ANY AND ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
INCLUDING ALL EXPRESS WARRANTIES AND ALL IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS, FOR A PARTICULAR PURPOSE. IN NO EVENT
SHALL SONY BE LIABLE FOR ANY DAMAGES OF ANY KIND, INCLUDING BUT NOT
LIMITED TO SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES RESULTING FROM
LOSS OF USE, DATA OR LOSS OF ANY PAST, PRESENT, OR PROSPECTIVE PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIENCE OR OTHER TORTIOUS ACTION, 
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS 
SOFTWARE.
*****************************************************************/

#include <stdio.h>
#include "Xpexlibint.h"
#include "Xpexlib.h"
#include <X11/extensions/Xext.h>
#include "extutil.h"
#include "Xpextutil.h"

static int size_of_rdr_attrs();
static void pack_rdr_attrs();
static void unpack_rdr_attrs();

XpexRenderer
XpexCreateRenderer(dpy, drawable, valuemask, values)
	Display  *dpy;
	Drawable  drawable;
	XpexBitmask valuemask;
	XpexRendererAttributes  *values;
{
	register 	XExtDisplayInfo *i = XpexFindDisplay(dpy);
	register	pexCreateRendererReq	*req;

	pexRenderer rdr_id;
    int	extra = 0;

	XpexCheckExtension(dpy,i,0);

	if (valuemask) {
		extra = size_of_rdr_attrs(valuemask, values);
	}
	LockDisplay(dpy);
	XpexGetReqExtra(CreateRenderer, dpy, i, extra, req);
	req->fpFormat = XpexGetFloatingPointFormat(dpy);
	req->drawable = drawable;
	req->rdr = rdr_id = XAllocID(dpy);
	req->itemMask = valuemask;
	if (extra) {
    	pack_rdr_attrs(valuemask, values, (char *)(req + 1));
	}
	UnlockDisplay(dpy);
	SyncHandle();

	return rdr_id;
}

void
XpexFreeRenderer(dpy, rdr_id)
	Display *dpy;
	XpexRenderer rdr_id;
{
	register XExtDisplayInfo *i = XpexFindDisplay(dpy);
	register pexFreeRendererReq *req;

	XpexSimpleCheckExtension(dpy,i);
	LockDisplay(dpy);
	XpexGetResReq(FreeRenderer, dpy, i, rdr_id, req);
	UnlockDisplay(dpy);
	SyncHandle();
}

void
XpexChangeRenderer(dpy, rdr_id, valueMask, values)
	Display *dpy;
	XpexRenderer rdr_id;
	XpexBitmask valueMask;
	XpexRendererAttributes *values;
{
	register XExtDisplayInfo *i = XpexFindDisplay(dpy);
	register pexChangeRendererReq *req;
    int extra = 0;

	XpexSimpleCheckExtension(dpy,i);

	if (valueMask) {
    	extra = size_of_rdr_attrs(valueMask, values);
	}
	LockDisplay(dpy);
	XpexGetReqExtra(ChangeRenderer, dpy, i, extra, req);
	req->rdr = rdr_id;
	req->itemMask = valueMask;
	if (extra) {
    	pack_rdr_attrs(valueMask, values, (char *)(req + 1));
	}
	UnlockDisplay(dpy);
	SyncHandle();
}

Status
XpexGetRendererAttributes(dpy, rdr_id, valueMask, values)
	register Display *dpy;
	XpexRenderer rdr_id;
	XpexBitmask valueMask;
	XpexRendererAttributes *values; /* RETURN */
{
	char *list;
	int bytes_unpacked;

	if (!_XpexGetRendererAttributes(dpy,rdr_id,valueMask,&list)) {
		return 0;
	}
	unpack_rdr_attrs(list,valueMask,values,&bytes_unpacked);

	if (list) {
		XFree(list);
	}
	return 1;
}

Status
_XpexGetRendererAttributes(dpy, rdr_id, valueMask, list)
	register Display *dpy;
	XpexRenderer rdr_id;
	XpexBitmask valueMask;
	char **list; /* RETURN */
{
	register	XExtDisplayInfo	*i = XpexFindDisplay(dpy);
	register	pexGetRendererAttributesReq		*req;
				pexGetRendererAttributesReply	reply;
	Status		status = 1;
	unsigned	size;

	XpexCheckExtension(dpy,i,0);
	LockDisplay(dpy);
	XpexGetReq(GetRendererAttributes, dpy, i, req);
	req->rdr = rdr_id;
	req->itemMask = valueMask;
	if( !_XReply(dpy, (xReply *)&reply, 0, xFalse)) {
		UnlockDisplay(dpy);
		SyncHandle();
		return 0;
	}
	if ((size = reply.length * sizeof(CARD32)) > 0) {
		if (*list = (char *) Xmalloc(size)) {
			_XRead(dpy, list, (long) size);
		} else {
			status = 0;
			_XpexClearReply(dpy, reply.length);
		}
	}
	UnlockDisplay(dpy);
	SyncHandle();

	return status;
}

Status
XpexGetRendererDynamics(dpy, rdr_id, dynamics)
	Display *dpy;
	XpexRenderer rdr_id;
	XpexRendererDynamics *dynamics; /* RETURN */
{
	register	XExtDisplayInfo *i = XpexFindDisplay(dpy);
	register	pexGetRendererDynamicsReq	*req;
				pexGetRendererDynamicsReply	reply;


	XpexCheckExtension(dpy,i,0);
	LockDisplay(dpy);
	XpexGetReq(GetRendererDynamics, dpy, i, req);
	req->id = rdr_id;
	if (!_XReply(dpy, (xReply *)&reply, 0, xFalse)) {
		UnlockDisplay(dpy)
		SyncHandle();
		return 0;
	}
	dynamics->tables = (unsigned long) reply.tables;
	dynamics->namesets = (unsigned long) reply.namesets;
	dynamics->attributes = (unsigned long) reply.attributes;
	UnlockDisplay(dpy);
	SyncHandle();

	return 1;
}

void
XpexBeginRendering(dpy, rdr_id, drawable_id)
	Display *dpy;
	XpexRenderer rdr_id;
	Drawable drawable_id;
{
	register	XExtDisplayInfo *i = XpexFindDisplay(dpy);
	register	pexBeginRenderingReq	*req;

	XpexSimpleCheckExtension(dpy,i);
	LockDisplay(dpy);
	XpexGetReq(BeginRendering, dpy, i, req);
	req->rdr = rdr_id;
	req->drawable = drawable_id;
	UnlockDisplay(dpy);
	SyncHandle();
}

void
XpexEndRendering(dpy, rdr_id, flush_flag)
	Display  *dpy;
	XpexRenderer  rdr_id;
	XpexSwitch  flush_flag;
{
	register	XExtDisplayInfo *i = XpexFindDisplay(dpy);
	register	pexEndRenderingReq	*req;

	XpexSimpleCheckExtension(dpy,i);
	LockDisplay(dpy);
	XpexGetReq(EndRendering,dpy, i, req);
	req->rdr = rdr_id;
	req->flushFlag = (pexSwitch) flush_flag;
	UnlockDisplay(dpy);
	SyncHandle();
}

void
XpexBeginStructure(dpy, rdr_id, s_id)
	Display *dpy;
	XpexRenderer rdr_id;
	XpexStructure s_id;
{
	register	XExtDisplayInfo *i = XpexFindDisplay(dpy);
	register	pexBeginStructureReq	*req;

	XpexSimpleCheckExtension(dpy,i);
	LockDisplay(dpy);
	XpexGetReq(BeginStructure,dpy, i, req);
	req->rdr = rdr_id;
	req->sid = s_id;
	UnlockDisplay(dpy);
	SyncHandle();
}

void
XpexEndStructure(dpy, rdr_id)
	Display *dpy;
	XpexRenderer  rdr_id;
{
	register	XExtDisplayInfo *i = XpexFindDisplay(dpy);
	register	pexEndStructureReq	*req;

	XpexSimpleCheckExtension(dpy,i);
	LockDisplay(dpy);
	XpexGetResReq(EndStructure,dpy, i, rdr_id, req);
	UnlockDisplay(dpy);
	SyncHandle();
}

void
XpexRenderOutputCommands(dpy, rdr_id)
	register Display *dpy;
	XpexRenderer rdr_id;
{
	register	XExtDisplayInfo *i = XpexFindDisplay(dpy);
	register	pexRenderOutputCommandsReq	*req;

	XpexSimpleCheckExtension(dpy,i);
	LockDisplay(dpy);
	XpexGetReq(RenderOutputCommands,dpy,i,req);
	req->fpFormat = XpexGetFloatingPointFormat(dpy);
	req->rdr = rdr_id;
	req->numCommands = 0;
	UnlockDisplay(dpy);
	/* SyncHandle();  Don't Sync */
}

void 
XpexRenderNetwork(dpy, rdr_id, drawable_id, s_id)
	register Display *dpy;
	XpexRenderer rdr_id;
	Drawable drawable_id;
	XpexStructure s_id;
{
	register	XExtDisplayInfo *i = XpexFindDisplay(dpy);
	register	pexRenderNetworkReq		*req;

	XpexSimpleCheckExtension(dpy,i);
	LockDisplay(dpy);
	XpexGetReq(RenderNetwork, dpy, i, req);
	req->rdr = rdr_id;
	req->drawable = drawable_id;
	req->sid = s_id;
	UnlockDisplay(dpy);
	SyncHandle();
}

static int
size_of_rdr_attrs(valueMask, values)
	XpexBitmask valueMask;
	XpexRendererAttributes *values;
{
    int size = 0;

	if (valueMask & PEXRDPipelineContext) {
    	size += sizeof(pexPC);
	}
	if (valueMask & PEXRDCurrentPath) {
    	size += sizeof(CARD32) +
		(values->current_path.num_refs * sizeof(pexElementRef));
	}
    if (valueMask & PEXRDMarkerBundle) {
    	size += sizeof(CARD32);
	}
    if (valueMask & PEXRDTextBundle) {
    	size += sizeof(CARD32);
	}
    if (valueMask & PEXRDLineBundle) {
    	size += sizeof(CARD32);
	}
    if (valueMask & PEXRDInteriorBundle) {
    	size += sizeof(CARD32);
	}
    if (valueMask & PEXRDEdgeBundle) {
    	size += sizeof(CARD32);
	}
    if (valueMask & PEXRDViewTable) {
    	size += sizeof(CARD32);
	}
    if (valueMask & PEXRDColourTable) {
    	size += sizeof(CARD32);
	}
    if (valueMask & PEXRDDepthCueTable) {
    	size += sizeof(CARD32);
	}
    if (valueMask & PEXRDLightTable) {
    	size += sizeof(CARD32);
	}
    if (valueMask & PEXRDPatternTable) {
    	size += sizeof(CARD32);
	}
    if (valueMask & PEXRDTextFontTable) {
    	size += sizeof(CARD32);
	}
    if (valueMask & PEXRDHighlightIncl) {
    	size += sizeof(CARD32);
	}
    if (valueMask & PEXRDHighlightExcl) {
    	size += sizeof(CARD32);
	}
    if (valueMask & PEXRDInvisibilityIncl) {
    	size += sizeof(CARD32);
	}
    if (valueMask & PEXRDInvisibilityExcl) {
    	size += sizeof(CARD32);
	}
    if (valueMask & PEXRDRendererState) {
    	size += sizeof(CARD32);
	}
    if (valueMask & PEXRDHlhsrMode) {
    	size += sizeof(CARD32);
	}
    if (valueMask & PEXRDNpcSubvolume) {
    	size += sizeof(pexNpcSubvolume);
	}
    if (valueMask & PEXRDViewport) {
    	size += sizeof(pexViewport);
	}
    if (valueMask & PEXRDClipList) {
    	size += sizeof(CARD32) +
		values->clip_list.num_rects * sizeof(pexDeviceRect);
	}
    return size;
}

static void
pack_rdr_attrs(valueMask, values, list)
	XpexBitmask valueMask;
	XpexRendererAttributes *values; 
	char *list; /* RETURN */
{
	CARD32 *d;
	if (valueMask & XpexRD_PipelineContext) {
		d = (CARD32 *) list;
		*d = values->pc_id;
		list += sizeof(CARD32);
	}
	if (valueMask & XpexRD_CurrentPath) {
		int n = values->current_path.num_refs;
		int size = n * sizeof(pexElementRef);
		d = (CARD32 *) list;
		*d = (CARD32) n;
		list += sizeof(CARD32); 

		if (size) {
			bcopy((char *)(values->current_path.elem_refs),list, size);
			list += size;
		}
	}
	if (valueMask & XpexRD_MarkerBundle) {
		d = (CARD32 *) list;
		*d = values->marker_bundle;
		list += sizeof(CARD32);
	}
	if (valueMask & XpexRD_TextBundle) {
		d = (CARD32 *) list;
		*d = values->text_bundle;
		list += sizeof(CARD32);
	}
	if (valueMask & XpexRD_LineBundle) {
		d = (CARD32 *) list;
		*d = values->line_bundle;
		list += sizeof(CARD32);
	}
	if (valueMask & XpexRD_InteriorBundle) {
		d = (CARD32 *) list;
		*d = values->interior_bundle;
		list += sizeof(CARD32);
	}
	if (valueMask & XpexRD_EdgeBundle) {
		d = (CARD32 *) list;
		*d = values->edge_bundle;
		list += sizeof(CARD32);
	}
	if (valueMask & XpexRD_ViewTable) {
		d = (CARD32 *) list;
		*d = values->view_table;
		list += sizeof(CARD32);
	}
	if (valueMask & XpexRD_ColorTable) {
		d = (CARD32 *) list;
		*d = values->color_table;
		list += sizeof(CARD32);
	}
	if (valueMask & XpexRD_DepthCueTable) {
		d = (CARD32 *) list;
		*d = values->depth_cue_table;
		list += sizeof(CARD32);
	}
	if (valueMask & XpexRD_LightTable) {
		d = (CARD32 *) list;
		*d = values->light_table;
		list += sizeof(CARD32);
	}
	if (valueMask & XpexRD_ColorApproxTable) {
		d = (CARD32 *) list;
		*d = values->color_approx_table;
		list += sizeof(CARD32);
	}
	if (valueMask & XpexRD_PatternTable) {
		d = (CARD32 *) list;
		*d = values->pattern_table;
		list += sizeof(CARD32);
	}
	if (valueMask & XpexRD_TextFontTable) {
		d = (CARD32 *) list;
		*d = values->text_font_table;
		list += sizeof(CARD32);
	}
	if (valueMask & XpexRD_HighlightIncl) {
		d = (CARD32 *) list;
		*d = values->highlight_incl;
		list += sizeof(CARD32);
	}
	if (valueMask & XpexRD_HighlightExcl) {
		d = (CARD32 *) list;
		*d = values->highlight_excl;
		list += sizeof(CARD32);
	}
	if (valueMask & XpexRD_InvisibilityIncl) {
		d = (CARD32 *) list;
		*d = values->invis_incl;
		list += sizeof(CARD32);
	}
	if (valueMask & XpexRD_InvisibilityExcl) {
		d = (CARD32 *) list;
		values->invis_excl = *((CARD32 *) list);
		list += sizeof(CARD32);
	}
	if (valueMask & XpexRD_RendererState) {
		CARD16 *v = (CARD16 *) list;
		*v = (CARD16) (values->renderer_state);
		list += sizeof(CARD32);
	}
	if (valueMask & XpexRD_HlhsrMode) {
		INT16 *v = (INT16 *) list;	
		*v = (INT16) values->hlhsr_mode;
		list += sizeof(CARD32);
	}
	if (valueMask & XpexRD_NpcSubvolume) {
		bcopy((char *)(values->npc_subvolume),(char *) list,
		sizeof(pexNpcSubvolume));
		list += sizeof(pexNpcSubvolume);
	}
	if (valueMask & XpexRD_Viewport) {
		bcopy((char *)(values->viewport),(char *)list, sizeof(pexViewport));
		list += sizeof(pexViewport);
	}
	if (valueMask & XpexRD_ClipList) {
		int n = values->clip_list.num_rects;
		int size = n * sizeof(pexDeviceRect);
		d = (CARD32 *) list;
		*d = (CARD32) n;
		list += sizeof(CARD32);
		if (size) {
			bcopy((char *)(values->clip_list.device_rect),(char *)list,size);
		}
		list += size;
	}
}

static void
unpack_rdr_attrs(list, valueMask, values, length)
	char *list;
	XpexBitmask valueMask;
	XpexRendererAttributes *values; /* RETURN */
	int *length; /* RETURN */
{
	*length = 0;

	if (valueMask & XpexRD_PipelineContext) {
		values->pc_id = *((CARD32 *) list);
		*length += sizeof(CARD32);
		list += sizeof(CARD32);
	}
	if (valueMask & XpexRD_CurrentPath) {

		int n =
		values->current_path.num_refs = (int) *((CARD32 *) list);
		int size = n * sizeof(pexElementRef);

		list += sizeof(CARD32);

		if (n) {
			values->current_path.elem_refs =
			(XpexElementRef *) Xmalloc((unsigned) size);

			bcopy(list, (char *)(values->current_path.elem_refs), size);
		}
		list += size;
		*length += sizeof(CARD32) + size;
	}
	if (valueMask & XpexRD_MarkerBundle) {
		values->marker_bundle = *((CARD32 *) list);
		*length += sizeof(CARD32);
		list += sizeof(CARD32);
	}
	if (valueMask & XpexRD_TextBundle) {
		values->text_bundle = *((CARD32 *) list);
		*length += sizeof(CARD32);
		list += sizeof(CARD32);
	}
	if (valueMask & XpexRD_LineBundle) {
		values->line_bundle = *((CARD32 *) list);
		*length += sizeof(CARD32);
		list += sizeof(CARD32);
	}
	if (valueMask & XpexRD_InteriorBundle) {
		values->interior_bundle = *((CARD32 *) list);
		*length += sizeof(CARD32);
		list += sizeof(CARD32);
	}
	if (valueMask & XpexRD_EdgeBundle) {
		values->edge_bundle = *((CARD32 *) list);
		*length += sizeof(CARD32);
		list += sizeof(CARD32);
	}
	if (valueMask & XpexRD_ViewTable) {
		values->view_table = *((CARD32 *) list);
		*length += sizeof(CARD32);
		list += sizeof(CARD32);
	}
	if (valueMask & XpexRD_ColorTable) {
		values->color_table = *((CARD32 *) list);
		*length += sizeof(CARD32);
		list += sizeof(CARD32);
	}
	if (valueMask & XpexRD_DepthCueTable) {
		values->depth_cue_table = *((CARD32 *) list);
		*length += sizeof(CARD32);
		list += sizeof(CARD32);
	}
	if (valueMask & XpexRD_LightTable) {
		values->light_table = *((CARD32 *) list);
		*length += sizeof(CARD32);
		list += sizeof(CARD32);
	}
	if (valueMask & XpexRD_ColorApproxTable) {
		values->color_approx_table = *((CARD32 *) list);
		*length += sizeof(CARD32);
		list += sizeof(CARD32);
	}
	if (valueMask & XpexRD_PatternTable) {
		values->pattern_table = *((CARD32 *) list);
		*length += sizeof(CARD32);
		list += sizeof(CARD32);
	}
	if (valueMask & XpexRD_TextFontTable) {
		values->text_font_table = *((CARD32 *) list);
		*length += sizeof(CARD32);
		list += sizeof(CARD32);
	}
	if (valueMask & XpexRD_HighlightIncl) {
		values->highlight_incl = *((CARD32 *) list);
		*length += sizeof(CARD32);
		list += sizeof(CARD32);
	}
	if (valueMask & XpexRD_HighlightExcl) {
		values->highlight_excl = *((CARD32 *) list);
		*length += sizeof(CARD32);
		list += sizeof(CARD32);
	}
	if (valueMask & XpexRD_InvisibilityIncl) {
		values->invis_incl = *((CARD32 *) list);
		*length += sizeof(CARD32);
		list += sizeof(CARD32);
	}
	if (valueMask & XpexRD_InvisibilityExcl) {
		values->invis_excl = *((CARD32 *) list);
		*length += sizeof(CARD32);
		list += sizeof(CARD32);
	}
	if (valueMask & XpexRD_RendererState) {
		values->hlhsr_mode = *((INT16 *) list);
		list += sizeof(CARD32);
		*length += sizeof(CARD32); /* 2 bytes of padding included */
	}
	if (valueMask & XpexRD_HlhsrMode) {
		values->renderer_state = *((CARD16 *) list);
		list += sizeof(CARD32);
		*length += sizeof(CARD32); /* 2 bytes of padding included */
	}
	if (valueMask & XpexRD_NpcSubvolume) {
		bcopy((char *)list,(char *)(values->npc_subvolume),
		sizeof(pexNpcSubvolume));
		list += sizeof(pexNpcSubvolume);
		*length += sizeof(pexNpcSubvolume); 
	}
	if (valueMask & XpexRD_Viewport) {
		bcopy((char *)list,(char *)(values->viewport),
		sizeof(pexViewport));
		list += sizeof(pexViewport);
		*length += sizeof(pexViewport); /* 2 bytes of padding included */
	}
	if (valueMask & XpexRD_ClipList) {
		int n = values->clip_list.num_rects = (int) *((CARD32 *) list);
		int size = n * sizeof(pexDeviceRect);
		list += sizeof(CARD32);
		if (values->clip_list.device_rect = 
		(XpexDeviceRect *) Xmalloc(size)) {
			bcopy((char *)list,(char *)(values->clip_list.device_rect),
			size);
		}
		list += size;
		*length += sizeof(CARD32) + size;
	}
}

