/* $Header: pexdpyinfo.c,v 2.1 91/07/11 12:36:42 sinyaw Exp $ */

/*****************************************************************
Copyright 1991 by Sony Microsystems Company, San Jose, California

					All Rights Reserved

Permission to use, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and
that both that copyright notice and this permission notice appear
in supporting documentation, and that the name of Sony not be used
in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.

SONY DISCLAIMS ANY AND ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
INCLUDING ALL EXPRESS WARRANTIES AND ALL IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS, FOR A PARTICULAR PURPOSE. IN NO EVENT
SHALL SONY BE LIABLE FOR ANY DAMAGES OF ANY KIND, INCLUDING BUT NOT
LIMITED TO SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES RESULTING FROM
LOSS OF USE, DATA OR LOSS OF ANY PAST, PRESENT, OR PROSPECTIVE PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIENCE OR OTHER TORTIOUS ACTION, 
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS 
SOFTWARE.
*****************************************************************/

/* pexdpyinfo.c */
#include <stdio.h>
#include <X11/Xlib.h>
#include <X11/Xatom.h>
#include <X11/Xutil.h>

#include "Xpexlib.h"
#include "Xpexutil.h"

#define BORDER_WIDTH   2

static Window openWindow();

static char *window_name = "";

static char *displayName = NULL;
static Display *dpy;
static Drawable drawableExample;
static Window drawable;
static int screen;

static XExtCodes *pex_codes;

InitX()
{
	dpy = XOpenDisplay(displayName);

	if (!dpy) {
		exit(-1);
	}

	screen = DefaultScreen(dpy);
	drawableExample = RootWindow(dpy,screen);

	XSynchronize(dpy, 1);
}

InitPEX()
{
	pex_codes = XpexInitialize(dpy);

	if (!pex_codes) {
		exit(-1);
	}
	XpexSetFloatingPointFormat(dpy, Xpex_Ieee_754_32);
}

main(argc, argv)
	int argc;
	char *argv[];
{
	InitX();
	InitPEX();

	dpy_ext_info();  
}

dpy_ext_info()
{
	static char *et_msg_str[27] = {
		"MarkerType(s)",
		"AtextStyle(s)",
		"InteriorStyle(s)",
		"HatchStyle(s)",
		"LineType(s)",
		"SurfaceEdgeTypes",
		"PickDeviceTypes", 
		"PolylineInterpMethod(s)",
		"CurveApproxMethod(s)",
		"ReflectionModel(s)",
		"SurfaceInterpMethod(s)",
		"SurfaceApproxMethod(s)",
		"ModelClipOperator(s)",
		"LightType(s)",
		"ColorType(s)",
		"FloatFormat(s)",
		"HlhsrMode(s)",
		"PromptEchoType(s)",
		"DisplayUpdateMode(s)",
		"ColorApproxType(s)",
		"ColorApproxModel(s)",
		"GDP(s)",
		"GDP3(s)",
		"GSE(s)",
		"TrimCurveApproxMethod(s)",
		"RenderingColourModel(s)",
		"ParaSurfCharacteristic(s)"
	};
	static char *idc_msg_str[34] = {
		NULL,
		"\tDitheringSupported: %d\n",
		"\tMaxEdgeWidth: %d\n",
		"\tMaxLineWidth: %d\n",
		"\tMaxMarkerSize: %d\n",
		"\tMaxModelClipPlanes: %d\n",
		"\tMaxNamesetNames: %d\n",
		"\tMaxNonAmbientLights: %d\n",
		"\tMaxNURBOrder: %d\n",
		"\tMaxTrimCurveOrder: %d\n",
		"\tMinEdgeWidth: %d\n",
		"\tMinLineWidth: %d\n",
		"\tMinMarkerSize: %d\n",
		"\tNominalEdgeWidth: %d\n",
		"\tNominalLineWidth: %d\n",
		"\tNominalMarkerSize: %d\n",
		"\tNumSupportedEdgeWidths: %d\n",
		"\tNumSupportedLineWidths: %d\n",
		"\tNumSupportedMarkerSizes: %d\n",
		"\tBestColorApproxValues: %d\n",
		"\tTransparencySupported: %d\n",
		"\tDoubleBufferSupported: %d\n",
		"\tChromaticityRedU: %.3f\n",
		"\tChromaticityRedV: %.3f\n",
		"\tLuminanceRed: %.3f\n",
		"\tChromaticityGreenU: %.3f\n",
		"\tChromaticityGreenU: %.3f\n",
		"\tLuminanceGreen: %.3f\n",
		"\tChromacityBlueU: %.3f\n",
		"\tChromacityBlueV: %.3f\n",
		"\tLuminanceBlue: %.3f\n",
		"\tChromacityWhiteU: %.3f\n",
		"\tChromacityWhiteV: %.3f\n",
		"\tLuminanceWhite: %.3f\n",
	};
	static short enum_types[27] = {
		XpexET_MarkerType,
		XpexET_ATextStyle,
		XpexET_InteriorStyle,
		XpexET_HatchStyle,
		XpexET_LineType,
		XpexET_SurfaceEdgeType,
		XpexET_PickDeviceType,
		XpexET_PolylineInterpMethod,
		XpexET_CurveApproxMethod,
		XpexET_ReflectionModel,
		XpexET_SurfaceInterpMethod,
		XpexET_SurfaceApproxMethod,
		XpexET_ModelClipOperator,
		XpexET_LightType,
		XpexET_ColorType,
		XpexET_FloatFormat,
		XpexET_HlhsrMode,
		XpexET_PromptEchoType,
		XpexET_DisplayUpdateMode,
		XpexET_ColorApproxType,
		XpexET_ColorApproxModel,
		XpexET_GDP,
		XpexET_GDP3,
		XpexET_GSE,
		XpexET_TrimCurveApproxMethod,
		XpexET_RenderingColourModel,
		XpexET_ParaSurfCharacteristics
	};
	XpexBitmask returnFormat = 0x0;
	XpexListOfEnumTypeDesc *info_list;
	int actual_count;
	register long i, j;
	XpexExtensionInfo info;
	XpexAllImpDepConstants idc;

	(void) fprintf(stderr, "Extension Information\n\n");

	(void) fprintf(stderr, 
	"\tmajor_opcode: %d\n", pex_codes->major_opcode);

	(void) fprintf(stderr, 
	"\tfirst_event: %d\n", pex_codes->first_event);

	(void) fprintf(stderr, 
	"\tfirst_error: %d\n", pex_codes->first_error);

	XpexGetExtensionInfo(dpy,5,0,&info);

	(void) fprintf(stderr, "\tversion number: %d.%d\n", 
	info.proto_major, info.proto_minor);

	(void) fprintf(stderr, 
	"\tvendor string: %s\n",info.vendor);

	(void) fprintf(stderr, 
	"\tvendor release number: %d\n", info.release_number);

	(void) fprintf(stderr, 
	"\tsubset info: %d\n\n", info.subset_info);

	returnFormat = 0x3;
	XpexGetEnumTypeInfo(dpy, drawableExample, 
	enum_types, 27, returnFormat, &info_list, &actual_count);

	(void) fprintf(stderr, "Enumerated Type Information\n\n");
	(void) fprintf(stderr, "\t%d Enumerated Type(s)\n\n", actual_count);

	for (j = 0; j < actual_count; j++) {
		(void) fprintf(stderr, "\tNumber of Supported %s: %d\n\n",
			et_msg_str[j], info_list[j].count);
		for (i = 0; i < info_list[j].count; i++) {
			(void) fprintf(stderr, "\t\t%s(%d)\n",
				info_list[j].desc[i].name, info_list[j].desc[i].index);
		}
		(void) fprintf(stderr, "\n");
	}

/* -------------------------------------------------------------- */
	XpexGetAllImpDepConstants(dpy, drawableExample, &idc);

	(void) fprintf(stderr, 
	"\nImplementation-Dependent Constants\n\n");

	(void) fprintf(stderr, idc_msg_str[XpexID_NominalLineWidth],
	idc.nominalLineWidth);

	(void) fprintf(stderr, idc_msg_str[XpexID_NumSupportedLineWidths],
	idc.numSupportedLineWidths);

	(void) fprintf(stderr, idc_msg_str[XpexID_MinLineWidth],
	idc.minLineWidth);

	(void) fprintf(stderr, idc_msg_str[XpexID_MaxLineWidth],
	idc.maxLineWidth);

	(void) fprintf(stderr, idc_msg_str[XpexID_NominalEdgeWidth],
	idc.nominalEdgeWidth);

	(void) fprintf(stderr, idc_msg_str[XpexID_NumSupportedEdgeWidths],
	idc.numSupportedEdgeWidths);

	(void) fprintf(stderr, idc_msg_str[XpexID_MinEdgeWidth],
	idc.minEdgeWidth);

	(void) fprintf(stderr, idc_msg_str[XpexID_MaxEdgeWidth],
	idc.maxEdgeWidth);

	(void) fprintf(stderr, idc_msg_str[XpexID_NominalMarkerSize],
	idc.nominalMarkerSize);

	(void) fprintf(stderr, idc_msg_str[XpexID_NumSupportedMarkerSizes],
	idc.numSupportedMarkerSizes);

	(void) fprintf(stderr, idc_msg_str[XpexID_MinMarkerSize],
	idc.minMarkerSize);

	(void) fprintf(stderr, idc_msg_str[XpexID_MaxMarkerSize],
	idc.maxMarkerSize);

	(void) fprintf(stderr, idc_msg_str[XpexID_ChromaticityRedU],
	idc.chromaticityRedU);

	(void) fprintf(stderr, idc_msg_str[XpexID_ChromaticityRedV],
	idc.chromaticityRedV);

	(void) fprintf(stderr, idc_msg_str[XpexID_LuminanceRed],
	idc.luminanceRed);

	(void) fprintf(stderr, idc_msg_str[XpexID_ChromaticityGreenU],
	idc.chromaticityGreenU);

	(void) fprintf(stderr, idc_msg_str[XpexID_ChromaticityGreenV],
	idc.chromaticityGreenV);

	(void) fprintf(stderr, idc_msg_str[XpexID_LuminanceGreen],
	idc.luminanceGreen);

	(void) fprintf(stderr, idc_msg_str[XpexID_ChromaticityBlueU],
	idc.chromaticityBlueU);

	(void) fprintf(stderr, idc_msg_str[XpexID_ChromaticityBlueV],
	idc.chromaticityBlueV);

	(void) fprintf(stderr, idc_msg_str[XpexID_LuminanceBlue],
	idc.luminanceBlue);

	(void) fprintf(stderr, idc_msg_str[XpexID_ChromaticityWhiteU],
	idc.chromaticityWhiteU);

	(void) fprintf(stderr, idc_msg_str[XpexID_ChromaticityWhiteV],
	idc.chromaticityWhiteV);

	(void) fprintf(stderr, idc_msg_str[XpexID_LuminanceWhite],
	idc.luminanceWhite);

	(void) fprintf(stderr, idc_msg_str[XpexID_MaxNameSetNames],
	idc.maxNamesetNames);

	(void) fprintf(stderr, idc_msg_str[XpexID_MaxModelClipPlanes],
	idc.maxModelClipPlanes);

	(void) fprintf(stderr, idc_msg_str[XpexID_TransparencySupported],
	idc.transparencySupported);

	(void) fprintf(stderr, idc_msg_str[XpexID_DitheringSupported],
	idc.ditheringSupported);

	(void) fprintf(stderr, idc_msg_str[XpexID_MaxNonAmbientLights],
	idc.maxNonAmbientLights);

	(void) fprintf(stderr, idc_msg_str[XpexID_MaxNURBOrder],
	idc.maxNurbOrder);

	(void) fprintf(stderr, idc_msg_str[XpexID_MaxTrimCurveOrder],
	idc.maxTrimCurveOrder);

	(void) fprintf(stderr, idc_msg_str[XpexID_BestColourApproximation],
	idc.bestColorApproxValues);

	(void) fprintf(stderr, idc_msg_str[XpexID_DoubleBufferingSupported],
	idc.doubleBufferingSupported);
}
