/* $Header: pexscinfo.c,v 2.1 91/07/11 12:36:52 sinyaw Exp $ */

/*****************************************************************
Copyright 1991 by Sony Microsystems Company, San Jose, California

					All Rights Reserved

Permission to use, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and
that both that copyright notice and this permission notice appear
in supporting documentation, and that the name of Sony not be used
in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.

SONY DISCLAIMS ANY AND ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
INCLUDING ALL EXPRESS WARRANTIES AND ALL IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS, FOR A PARTICULAR PURPOSE. IN NO EVENT
SHALL SONY BE LIABLE FOR ANY DAMAGES OF ANY KIND, INCLUDING BUT NOT
LIMITED TO SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES RESULTING FROM
LOSS OF USE, DATA OR LOSS OF ANY PAST, PRESENT, OR PROSPECTIVE PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIENCE OR OTHER TORTIOUS ACTION, 
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS 
SOFTWARE.
*****************************************************************/

/*
 *
 */
#include <stdio.h>
#include <X11/Xlib.h>
#include <X11/Xatom.h>
#include <X11/Xutil.h>

#include "Xpexlibint.h"
#include "Xpexlib.h"

static char *displayName = NULL;
static Display *dpy;

main(argc, argv)
	int argc;
	char *argv[];
{
	InitX();
	InitPEX();

	dpy_sc_info(); 
}

InitX()
{
	dpy = XOpenDisplay(displayName);

	if (!dpy) {
		exit(-1);
	}

	XSynchronize(dpy, 1);
}

InitPEX()
{
	XExtCodes *pex_codes;

	pex_codes = XpexInitialize(dpy);

	if (!pex_codes) {
		exit(-1);
	}
	XpexSetFloatingPointFormat(dpy, Xpex_Ieee_754_32);
}

#ifdef SHOW_BUG
dpy_sc_info() 
{
	static char *sc_msg_str[8] = {
		"Search Context Attributes\n\n",
		"\tsearch_pos: (%.1f, %.1f, %.1f)\n",
		"\tsearch_dist: %.1f\n",
		"\tsearch_ceiling: %d\n",
		"\tmodel_clip_flag: %d\n",
		"\tstart_path: %d\n",
		"\tnormal_list: %d\n",
		"\tinverted_list: %d\n"
	};
	XpexSC sc_id;
	XpexBitmask valueMask;
	XpexSCValues values;

	valueMask = 0x0;

	sc_id = XpexCreateSC(dpy, valueMask, &values);

	valueMask = XpexSC_All;

	XpexGetSC(dpy, sc_id, valueMask, &values);

	(void) fprintf(stderr, sc_msg_str[1],
	values.search_pos.x, values.search_pos.y, values.search_pos.z);

	(void) fprintf(stderr, sc_msg_str[2], values.search_dist);

	(void) fprintf(stderr, sc_msg_str[3], values.search_ceiling);

	(void) fprintf(stderr, sc_msg_str[4], values.model_clip_flag);

/* By default, The PEX-SI Server is return outrageously big values for the 
	start_path, normal_list, and inverted_list. */

	(void) fprintf(stderr, sc_msg_str[5], values.start_path.num_refs);

	(void) fprintf(stderr, sc_msg_str[6], values.normal_list.num_pairs);

	(void) fprintf(stderr, sc_msg_str[7], values.inverted_list.num_pairs);

	XpexFreeSC(dpy, sc_id);
}
#else /* !SHOW_BUG */
dpy_sc_info()
{
	static char *sc_msg_str[8] = {
		"Search Context Attributes\n\n",
		"\tsearch_pos: (%.1f, %.1f, %.1f)\n",
		"\tsearch_dist: %.1f\n",
		"\tsearch_ceiling: %d\n",
		"\tmodel_clip_flag: %s(%d)\n",
		"\tstart_path: %d\n",
		"\tnormal_list: %d\n",
		"\tinverted_list: %d\n"
	};
	XpexSC sc_id;
	XpexBitmask valueMask;
	XpexSCValues values;

	valueMask = XpexSC_Position |
		XpexSC_Distance |
		XpexSC_Ceiling |
		XpexSC_ModelClipFlag |
		XpexSC_StartPath |
		XpexSC_NormalList |
		XpexSC_InvertedList;

	values.search_pos.x = 0.;
	values.search_pos.y = 0.;
	values.search_pos.z = 0.;
	values.search_dist = 0.0;
	values.search_ceiling = 0;
	values.model_clip_flag = 0;
	values.start_path.num_refs = 0;
	values.normal_list.num_pairs = 0;
	values.inverted_list.num_pairs = 0;

	sc_id = XpexCreateSC(dpy, valueMask, &values);

	(void) fprintf(stderr, sc_msg_str[0]);

	valueMask = XpexSC_Position | 
		XpexSC_Distance |
		XpexSC_Ceiling |
		XpexSC_ModelClipFlag;

	XpexGetSC(dpy, sc_id, valueMask, &values);

	(void) fprintf(stderr, sc_msg_str[1],
	values.search_pos.x, values.search_pos.y, values.search_pos.z);

	(void) fprintf(stderr, sc_msg_str[2], values.search_dist);

	(void) fprintf(stderr, sc_msg_str[3], values.search_ceiling);

	(void) fprintf(stderr, sc_msg_str[4], 
		(values.model_clip_flag) ? "True" : "False", 
		values.model_clip_flag);

/* By default, The PEX-SI Server is return outrageously big values 
for the start_path, normal_list, and inverted_list. */

	(void) fprintf(stderr, sc_msg_str[5], values.start_path.num_refs);

	(void) fprintf(stderr, sc_msg_str[6], values.normal_list.num_pairs);

	(void) fprintf(stderr, sc_msg_str[7], values.inverted_list.num_pairs);

	XpexFreeSC(dpy, sc_id);
}
#endif /*  SHOW_BUG */
