/* $Header: pexwksinfo.c,v 2.1 91/07/11 12:36:56 sinyaw Exp $ */

/*****************************************************************
Copyright 1991 by Sony Microsystems Company, San Jose, California

					All Rights Reserved

Permission to use, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and
that both that copyright notice and this permission notice appear
in supporting documentation, and that the name of Sony not be used
in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.

SONY DISCLAIMS ANY AND ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
INCLUDING ALL EXPRESS WARRANTIES AND ALL IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS, FOR A PARTICULAR PURPOSE. IN NO EVENT
SHALL SONY BE LIABLE FOR ANY DAMAGES OF ANY KIND, INCLUDING BUT NOT
LIMITED TO SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES RESULTING FROM
LOSS OF USE, DATA OR LOSS OF ANY PAST, PRESENT, OR PROSPECTIVE PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIENCE OR OTHER TORTIOUS ACTION, 
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS 
SOFTWARE.
*****************************************************************/

/*
 *
 */
#include <stdio.h>
#include <X11/Xlib.h>
#include <X11/Xatom.h>
#include <X11/Xutil.h>

#include "Xpexlibint.h"
#include "Xpexlib.h"
#include "Xpexutil.h"

#define BORDER_WIDTH   2

static Window openWindow();

static char *window_name = "";

static char *displayName = NULL;
static Display *dpy;
static Drawable drawableExample;
static Window drawable;
static int screen;

static XpexLookupTable lineBundleLUT;
static XpexLookupTable markerBundleLUT;
static XpexLookupTable textBundleLUT;
static XpexLookupTable interiorBundleLUT;
static XpexLookupTable edgeBundleLUT;
static XpexLookupTable patternLUT;
static XpexLookupTable textFontLUT;
static XpexLookupTable colorLUT;
static XpexLookupTable lightLUT;
static XpexLookupTable depthCueLUT;
static XpexLookupTable colorApproxLUT;
static XpexRenderer rdr_id;
static XpexNameSet highlightIncl;
static XpexNameSet highlightExcl;
static XpexNameSet invisIncl;
static XpexNameSet invisExcl;
static XpexPhigsWks wks_id;

main(argc, argv)
	int argc;
	char *argv[];
{
	InitX();
	InitPEX();

	dpy_wks_info();
	dpy_pick_info();
	clean_up(); 
}

InitX()
{
	dpy = XOpenDisplay(displayName);

	if (!dpy) {
		exit(-1);
	}

	XSynchronize(dpy, 1);
	screen = DefaultScreen(dpy);
	drawableExample = RootWindow(dpy,screen);
	drawable = openWindow(50,50, 100, 100, 1); 
}

InitPEX()
{
	XExtCodes *pex_codes;

	pex_codes = XpexInitialize(dpy);

	if (!pex_codes) {
		exit(-1);
	}
	XpexSetFloatingPointFormat(dpy, Xpex_Ieee_754_32);
	InitResources();
}

InitResources()
{
	markerBundleLUT = XpexCreateMarkerBundleLUT(dpy, drawable);
	lineBundleLUT = XpexCreateLineBundleLUT(dpy, drawable);
	textBundleLUT = XpexCreateTextBundleLUT(dpy, drawable);
	interiorBundleLUT = XpexCreateInteriorBundleLUT(dpy, drawable);
	edgeBundleLUT = XpexCreateEdgeBundleLUT(dpy, drawable);
	patternLUT = XpexCreatePatternLUT(dpy, drawable);
	textFontLUT = XpexCreateTextFontLUT(dpy, drawable);
	colorLUT = XpexCreateColorLUT(dpy, drawable);
	lightLUT = XpexCreateLightLUT(dpy, drawable);
	depthCueLUT = XpexCreateDepthCueLUT(dpy, drawable);
	colorApproxLUT = XpexCreateColorApproxLUT(dpy, drawable);

	highlightIncl = XpexCreateNameSet(dpy);
	highlightExcl = XpexCreateNameSet(dpy);
	invisIncl = XpexCreateNameSet(dpy);
	invisExcl = XpexCreateNameSet(dpy);
}

static Window
openWindow(x, y, width, height, flag)
	int x, y;
	int width, height;
	int flag;
{
	XSetWindowAttributes theWindowAttributes;
	XSizeHints theSizeHints;
	unsigned long theWindowMask;
	Window theNewWindow;
	int theScreen = DefaultScreen(dpy);
	int theDepth = DefaultDepth(dpy, theScreen);

	theWindowAttributes.border_pixel = BlackPixel(dpy, theScreen);
	theWindowAttributes.background_pixel = WhitePixel(dpy, theScreen);
	theWindowAttributes.override_redirect = True;
	theWindowMask = CWBackPixel | CWBorderPixel | CWOverrideRedirect;

	theNewWindow = XCreateWindow(dpy,
		RootWindow(dpy, theScreen),
		x, y,
		width, height,
		BORDER_WIDTH,
		theDepth,
		InputOutput,
		CopyFromParent,
		theWindowMask,
		&theWindowAttributes);

	theSizeHints.flags = PPosition | PSize;
	theSizeHints.x = x;
	theSizeHints.y = y;
	theSizeHints.width = width;
	theSizeHints.height = height;

	XSetNormalHints(dpy, theNewWindow, &theSizeHints);

	XMapWindow(dpy, theNewWindow);

	XFlush(dpy);

	return(theNewWindow);
}

dpy_wks_info()
{
	static char *display_update_method_str[6] = {
		NULL,
		"VisualizeEach",  /* 1 */
		"VisualizeEasy",  /* 2 */
		"VisualizeNone", /* 3 */
		"SimulateSome",  /* 4 */
		"VisualizeWhenever" /* 5 */
	};

	static char *visual_state_str[3] = {
		"Correct",  /* 0 */
		"Deferred",  /* 1 */
		"Simulate" /* 2 */
	};

	XpexWksResources wks_res;
	XpexBitmask valuemask[2];
	XpexWksInfo info;

	wks_res.line_bundle = lineBundleLUT;
	wks_res.marker_bundle = markerBundleLUT;
	wks_res.text_bundle = textBundleLUT;
	wks_res.interior_bundle = interiorBundleLUT;
	wks_res.edge_bundle = edgeBundleLUT;
	wks_res.color_table = colorLUT;
	wks_res.depth_cue_table = depthCueLUT;
	wks_res.light_table = lightLUT;
	wks_res.color_approx_table = colorApproxLUT;
	wks_res.pattern_table = patternLUT;
	wks_res.text_font_table = textFontLUT;
	wks_res.highlight_incl = highlightIncl;
	wks_res.highlight_excl = highlightExcl;
	wks_res.invis_incl = invisIncl;
	wks_res.invis_excl = invisExcl;

	wks_id = XpexCreatePhigsWks(dpy, drawable, 
	&wks_res, Xpex_SingleBuffered);

	XpexSetDisplayUpdateMode(dpy, wks_id, Xpex_VisualizeEach);

	(void) fprintf(stderr, 
	"PHIGS Workstation Information (Wks Resource ID : %u)\n\n",
	wks_id);

	valuemask[0] = XpexPW_DisplayUpdate;
	valuemask[1] = 0x0;

	XpexGetWksInfo(dpy, wks_id, valuemask, &info);

	(void) fprintf(stderr, "\tDisplayUpdate: %d\n", 
	info.display_update);

	valuemask[0] = 
		XpexPW_VisualState |
		XpexPW_DisplaySurface |
		XpexPW_ViewUpdate |
		XpexPW_DefinedViews |
		XpexPW_WksUpdate |
		XpexPW_ReqNpcSubvolume |
		XpexPW_CurNpcSubvolume |
		XpexPW_HlhsrUpdate |
		XpexPW_ReqHlhsrMode |
		XpexPW_CurHlhsrMode;

	(void) fprintf(stderr, "\tVisualState: %d\n", 
	info.visual_state);

	(void) fprintf(stderr, "\tDisplaySurface: %s(%d)\n",
	((info.display_surface) ? "NotEmpty" : "Empty"),
	info.display_surface);

	(void) fprintf(stderr, "\tViewUpdate: %s(%d)\n",
	((info.view_update) ? "Pending" : "NotPending"),
	info.view_update);

	(void) fprintf(stderr, "\tDefinedViews: %d\n",
	info.defined_views.num_indices);

	(void) fprintf(stderr, "\tWksUpdate: %s(%d)\n",
	((info.wks_update) ? "Pending" : "NotPending"),
	info.wks_update);

	(void) fprintf(stderr, 
		"\treq_NPC_subvolume: (%.1f, %.1f %1.f),(%.1f, %.1f %.1f)\n",
		info.req_NPC_subvolume.minval.x,
		info.req_NPC_subvolume.minval.y,
		info.req_NPC_subvolume.minval.z,
		info.req_NPC_subvolume.maxval.x,
		info.req_NPC_subvolume.maxval.y,
		info.req_NPC_subvolume.maxval.z);

	(void) fprintf(stderr, 
		"\tcur_NPC_subvolume: (%.1f, %.1f %1.f),(%.1f, %.1f %.1f)\n",
		info.cur_NPC_subvolume.minval.x,
		info.cur_NPC_subvolume.minval.y,
		info.cur_NPC_subvolume.minval.z,
		info.cur_NPC_subvolume.maxval.x,
		info.cur_NPC_subvolume.maxval.y,
		info.cur_NPC_subvolume.maxval.z);

	(void) fprintf(stderr,
		"\treq_wks_viewpt: (%d,%d,%.1f),(%d,%d,%.1f), %s(%d)\n",
		info.req_wks_viewpt.minval.x,
		info.req_wks_viewpt.minval.y,
		info.req_wks_viewpt.minval.z,
		info.req_wks_viewpt.maxval.x,
		info.req_wks_viewpt.maxval.y,
		info.req_wks_viewpt.maxval.z,
		((info.req_wks_viewpt.useDrawable) ? "True" : "False"),
		info.req_wks_viewpt.useDrawable);

	(void) fprintf(stderr,
		"\tcur_wks_viewpt: (%d,%d,%.1f),(%d,%d,%.1f), %s(%d)\n",
		info.cur_wks_viewpt.minval.x,
		info.cur_wks_viewpt.minval.y,
		info.cur_wks_viewpt.minval.z,
		info.cur_wks_viewpt.maxval.x,
		info.cur_wks_viewpt.maxval.y,
		info.cur_wks_viewpt.maxval.z,
		((info.cur_wks_viewpt.useDrawable) ? "True" : "False"),
		info.req_wks_viewpt.useDrawable);

	valuemask[0] = 
		XpexPW_Drawable | 
		XpexPW_MarkerBundle | 
		XpexPW_TextBundle | 
		XpexPW_LineBundle |
		XpexPW_InteriorBundle |
		XpexPW_EdgeBundle |
		XpexPW_ColorTable |
		XpexPW_DepthCueTable |
		XpexPW_LightTable |
		XpexPW_ColorApproxTable |
		XpexPW_PatternTable |
		XpexPW_TextFontTable |
		XpexPW_HighlightIncl |
		XpexPW_HighlightExcl |
		XpexPW_InvisibilityIncl |
		XpexPW_InvisibilityExcl |
		XpexPW_PostedStructures |
		XpexPW_NumPriorities |
		XpexPW_BufferUpdate;

	valuemask[1] = 
		XpexPW_ReqBufferMode |
		XpexPW_CurBufferMode;

	XpexGetWksInfo(dpy, wks_id, valuemask, &info);
	(void) fprintf(stderr, "\tdrawable_id: %u\n", info.drawable_id);
	(void) fprintf(stderr, "\tmarker_bundle: %u\n", info.marker_bundle);
	(void) fprintf(stderr, "\ttext_bundle: %u\n", info.text_bundle);
	(void) fprintf(stderr, "\tline_bundle: %u\n", info.line_bundle);
	(void) fprintf(stderr, "\tinterior_bundle: %u\n", info.interior_bundle);

	(void) fprintf(stderr, "\tedge_bundle: %u\n", info.edge_bundle);

	(void) fprintf(stderr, "\tcolor_table: %u\n", info.color_table);

	(void) fprintf(stderr, 
	"\tdepth_cue_table: %u\n", info.depth_cue_table);

	(void) fprintf(stderr, "\tlight_table: %u\n", info.light_table);

	(void) fprintf(stderr, 
	"\tcolor_approx_table: %u\n", info.color_approx_table);

	(void) fprintf(stderr, "\tpattern_table: %u\n", info.pattern_table);

	(void) fprintf(stderr, "\ttext_font_table: %u\n", info.text_font_table);
	(void) fprintf(stderr, "\thighlight_incl: %u\n", info.highlight_incl);
	(void) fprintf(stderr, "\thighlight_excl: %u\n", info.highlight_excl);
	(void) fprintf(stderr, "\tinvis_incl: %u\n", info.invis_incl);
	(void) fprintf(stderr, "\tinvis_excl: %u\n", info.invis_excl);
	(void) fprintf(stderr, "\tposted_structs: %d\n", 
	info.posted_structs.num_info);

	(void) fprintf(stderr, "\tnum_priorities: %d\n", 
	info.num_priorities);

	(void) fprintf(stderr, "\tbuffer_update: %s(%d)\n", 
	(info.buffer_update) ? "Pending" : "NotPending", 
	info.buffer_update);

	(void) fprintf(stderr, "\treq_buffer_mode: %s(%d)\n", 
	((info.req_buffer_mode) ? "Double" : "Single"), 
	info.req_buffer_mode);

	(void) fprintf(stderr, "\tcur_buffer_mode: %s(%d)\n", 
	((info.cur_buffer_mode) ? "Double" : "Single"),
	info.req_buffer_mode);

	dpy_dyn();
}

dpy_dyn()
{
	static char *dyn_str[18] = {
		"\tview_rep            : %s\n",
		"\tmarker_bundle       : %s\n",
		"\ttext_bundle         : %s\n",
		"\tline_bundle         : %s\n",
		"\tinterior_bundle     : %s\n",
		"\tedge_bundle         : %s\n",
		"\tcolor_table         : %s\n",
		"\tpattern_table       : %s\n",
		"\twks_transform       : %s\n",
		"\thighlight_filter    : %s\n",
		"\tinvisibility_filter : %s\n",
		"\thlhsr_mode          : %s\n",
		"\tstructure_modify    : %s\n",
		"\tpost_structure      : %s\n",
		"\tunpost_structure    : %s\n",
		"\tdelete_structure    : %s\n",
		"\treference_modify    : %s\n",
		"\tbuffer_modify       : %s\n"
	};
	static char *wks_dyn_str[3] = {
		"IMM",
		"IRG",
		"CBS"
	};
	XpexDynamics dynamics;

	(void) fprintf(stderr, "\nPHIGS Workstation Dynamics\n\n");

	XpexGetDynamics(dpy, drawable, &dynamics);

	(void) fprintf(stderr, dyn_str[0], wks_dyn_str[dynamics.view_rep]);
	(void) fprintf(stderr, dyn_str[1], wks_dyn_str[dynamics.marker_bundle]);
	(void) fprintf(stderr, dyn_str[2], wks_dyn_str[dynamics.text_bundle]);
	(void) fprintf(stderr, dyn_str[3], wks_dyn_str[dynamics.line_bundle]);
	(void) fprintf(stderr, dyn_str[4], wks_dyn_str[dynamics.interior_bundle]);
	(void) fprintf(stderr, dyn_str[5], wks_dyn_str[dynamics.edge_bundle]);
	(void) fprintf(stderr, dyn_str[6], wks_dyn_str[dynamics.color_table]);
	(void) fprintf(stderr, dyn_str[7], wks_dyn_str[dynamics.pattern_table]);
	(void) fprintf(stderr, dyn_str[8], wks_dyn_str[dynamics.wks_transform]);
	(void) fprintf(stderr, dyn_str[9], wks_dyn_str[dynamics.highlight_filter]);
	(void) fprintf(stderr, dyn_str[10], wks_dyn_str[dynamics.invisibility_filter]);
	(void) fprintf(stderr, dyn_str[11], wks_dyn_str[dynamics.hlhsr_mode]);
	(void) fprintf(stderr, dyn_str[12], wks_dyn_str[dynamics.structure_modify]);
	(void) fprintf(stderr, dyn_str[13], wks_dyn_str[dynamics.post_structure]);
	(void) fprintf(stderr, dyn_str[14], wks_dyn_str[dynamics.unpost_structure]);
	(void) fprintf(stderr, dyn_str[15], wks_dyn_str[dynamics.delete_structure]);
	(void) fprintf(stderr, dyn_str[16], wks_dyn_str[dynamics.reference_modify]);
	(void) fprintf(stderr, dyn_str[17], wks_dyn_str[dynamics.buffer_modify]);

}

dpy_pick_info()
{
	XpexPickMeasure pm_id;
	XpexBitmask valueMask;
	XpexPickDevice values;

	valueMask = 
		XpexPD_PickStatus |
		XpexPD_PickPath |
		XpexPD_PickPathOrder |
		XpexPD_PickIncl |
		XpexPD_PickExcl |
		/* XpexPD_PickDataRec | */
		XpexPD_PickPromptEchoType |
		XpexPD_PickEchoVolume |
		XpexPD_PickEchoSwitch;

	XpexGetPickDevice(dpy, wks_id, 
	Xpex_PickDeviceDC_HitBox, valueMask, &values);

	(void) fprintf(stderr, "\nPick Device Descriptors\n\n");

	(void) fprintf(stderr, "\tPickStatus: %s(%d)\n",
	((values.pick_status) ? "Ok" : "NoPick"), values.pick_status);

	(void) fprintf(stderr, "\tPickPath: %d\n", 
	values.pick_path.num_refs);

	(void) fprintf(stderr, "\tPickPathOrder: %s(%d)\n", 
	((values.pick_path_order) ? "BottomFirst" : "TopFirst"),
	values.pick_path_order);

	(void) fprintf(stderr, "\tPickIncl: %u\n", values.pick_incl);
	(void) fprintf(stderr, "\tPickExcl: %u\n", values.pick_excl);

	(void) fprintf(stderr, 
		"\tPickDeviceDC_HitBox: (%d, %d, %f)\n",
		values.pick_data_rec.data_rec1.position.x,
		values.pick_data_rec.data_rec1.position.y,
		values.pick_data_rec.data_rec1.distance);

	(void) fprintf(stderr, "\tPickPromptEchoType: %d\n", 
	values.prompt_echo_type);

	(void) fprintf(stderr, 
		"\tPickEchoVolume: { (%d,%d), (%d, %d), %s(%d) }\n",
		values.echo_volume.minval.x, values.echo_volume.minval.y,
		values.echo_volume.maxval.x, values.echo_volume.maxval.y,
		((values.echo_volume.useDrawable) ? "True" : "False"),
		values.echo_volume.useDrawable);

	(void) fprintf(stderr, "\tEchoSwitch: %s(%d)\n", 
	((values.prompt_echo_type) ? "Echo" : "NoEcho"),
	values.prompt_echo_type);

	pm_id = XpexCreatePickMeasure(dpy, 
	wks_id, Xpex_PickDeviceDC_HitBox);
}

clean_up()
{
	XpexFreePhigsWks(dpy, wks_id); 
	XpexFreeLookupTable(dpy, lineBundleLUT);
	XpexFreeLookupTable(dpy, markerBundleLUT);
	XpexFreeLookupTable(dpy, textBundleLUT);
	XpexFreeLookupTable(dpy, interiorBundleLUT);
	XpexFreeLookupTable(dpy, edgeBundleLUT);
	XpexFreeLookupTable(dpy, patternLUT);
	XpexFreeLookupTable(dpy, colorLUT);
	XpexFreeLookupTable(dpy, textFontLUT);
	XpexFreeLookupTable(dpy, lightLUT);
	XpexFreeLookupTable(dpy, depthCueLUT);
	XpexFreeLookupTable(dpy, colorApproxLUT);
	XpexFreeNameSet(dpy, highlightIncl);
	XpexFreeNameSet(dpy, highlightExcl);
	XpexFreeNameSet(dpy, invisIncl);
	XpexFreeNameSet(dpy, invisExcl);
}
